# -*- coding: utf-8 -*-
#
# libcaca       Colour ASCII-Art library
#               Python language bindings
# Copyright (c) 2010 Alex Foulon <alxf@lavabit.com>
#               All Rights Reserved
#
# This library is free software. It comes without any warranty, to
# the extent permitted by applicable law. You can redistribute it
# and/or modify it under the terms of the Do What the Fuck You Want
# to Public License, Version 2, as published by Sam Hocevar. See
# http://www.wtfpl.net/ for more details.
#

""" Libcaca Python bindings """

import ctypes

from caca import _lib
from caca.canvas import _Canvas

class _Dither(object):
    """ Model for Dither object.
    """
    def __init__(self):
        self._dither = 0

    def from_param(self):
        """ Required by ctypes module to call object as parameter of
            a C function.
        """
        return self._dither

    def __del__(self):
        if self._dither > 0:
            self._free()

    def __str__(self):
        return "<CacaDither>"

    def _free(self):
        """ Free a libcaca dither.
        """
        _lib.caca_free_dither.argtypes = [_Dither]
        _lib.caca_free_dither.restype = ctypes.c_int

        return _lib.caca_free_dither(self)

class Dither(_Dither):
    """ Dither object, methods are libcaca functions with caca_dither_t as first
        argument.
    """
    def __init__(self, bpp, width, height, pitch, rmask, gmask, bmask, amask):
        """ Dither constructor

            bpp     -- bitmap depth in bits per pixels
            width   -- bitmap width in pixels
            height  -- bitmap height in pixels
            pitch   -- bitmap pitch in bytes
            rmask   -- bitmask for red values
            gmask   -- bitmask for green values
            bmask   -- bitmask for blue values
            amask   -- bitmask for alpha values
        """
        _lib.caca_create_dither.argtypes = [
            ctypes.c_int, ctypes.c_int, ctypes.c_int, ctypes.c_int,
            ctypes.c_uint, ctypes.c_uint, ctypes.c_uint, ctypes.c_uint,
        ]

        self._dither = _lib.caca_create_dither(bpp, width, height, pitch,
                                               rmask, gmask, bmask, amask)

        if self._dither == 0:
            raise DitherError("Failed to create dither object")

    def set_palette(self, red, green, blue, alpha):
        """ Set the palette of an 8 bits per pixel bitmap. Values should be
        between 0 and 4095 (0xfff).

            red     -- array of 256 red values
            green   -- array of 256 green values
            blue    -- array of 256 blue values
            alpha   -- array of 256 alpha values
        """
        raise DitherError("Not implemented")

    def set_brightness(self, brightness):
        """ Set the brightness of the dither object.

            brightness  -- brightness value
        """
        if isinstance(brightness, int):
            brightness = float(brightness)

        _lib.caca_set_dither_brightness.argtypes = [_Dither, ctypes.c_float]
        _lib.caca_set_dither_brightness.restype  = ctypes.c_int

        return _lib.caca_set_dither_brightness(self, brightness)

    def get_brightness(self):
        """ Get the brightness of the dither object.
        """
        _lib.caca_get_dither_brightness.argtypes = [_Dither]
        _lib.caca_get_dither_brightness.restype  = ctypes.c_float

        return _lib.caca_get_dither_brightness(self)

    def set_gamma(self, gamma):
        """ Set the gamma of the dither object. A negative value causes colour
        inversion.

            gamma -- gamma value
        """
        if isinstance(gamma, int):
            gamma = float(gamma)

        _lib.caca_set_dither_gamma.argtypes = [_Dither, ctypes.c_float]
        _lib.caca_set_dither_gamma.restype  = ctypes.c_int

        return _lib.caca_set_dither_gamma(self, gamma)

    def get_gamma(self):
        """ Get the gamma of the dither object.
        """
        _lib.caca_get_dither_gamma.argtypes = [_Dither]
        _lib.caca_get_dither_gamma.restype  = ctypes.c_float

        return _lib.caca_get_dither_gamma(self)

    def set_contrast(self, contrast):
        """ Set the contrast of dither.

            contrast -- contrast value
        """
        if isinstance(contrast, int):
            contrast = float(contrast)

        _lib.caca_set_dither_contrast.argtypes = [_Dither, ctypes.c_float]
        _lib.caca_set_dither_contrast.restype  = ctypes.c_int

        return _lib.caca_set_dither_contrast(self, contrast)

    def get_contrast(self):
        """ Get the contrast of the dither object.
        """
        _lib.caca_get_dither_contrast.argtypes = [_Dither]
        _lib.caca_get_dither_contrast.restype  = ctypes.c_float

        return _lib.caca_get_dither_contrast(self)

    def set_antialias(self, value):
        """ Set dither antialiasing.

            value -- A string describing the antialiasing method that will
                     be used for the dithering.

                     + "none": no antialiasing
                     + "prefilter" or "default": simple prefilter antialiasing. (default)
        """
        _lib.caca_set_dither_antialias.argtypes = [_Dither, ctypes.c_char_p]
        _lib.caca_set_dither_antialias.restype  = ctypes.c_int

        return _lib.caca_set_dither_antialias(self, value)

    def get_antialias(self):
        """ Return the dither's current antialiasing method.
        """
        _lib.caca_get_dither_antialias.argtypes = [_Dither]
        _lib.caca_get_dither_antialias.restype  = ctypes.c_char_p

        return _lib.caca_get_dither_antialias(self)

    def get_antialias_list(self):
        """ Get available antialiasing methods.
        """
        lst = []

        _lib.caca_get_dither_antialias_list.argtypes = [_Dither]
        _lib.caca_get_dither_antialias_list.restype  = ctypes.POINTER(ctypes.c_char_p)

        for item in _lib.caca_get_dither_antialias_list(self):
            if item is not None and item != "":
                lst.append(item)
            else:
                #memory occurs otherwise
                break

        return lst

    def set_color(self, value):
        """ Choose colours used for dithering.

            value -- A string describing the colour set that will be
                     used for the dithering.

                     + "mono": use light gray on a black background
                     + "gray": use white and two shades of gray on a black background
                     + "8": use the 8 ANSI colours on a black background
                     + "16": use the 16 ANSI colours on a black background
                     + "fullgray": use black, white and two shades of gray
                       for both the characters and the background
                     + "full8": use the 8 ANSI colours for both the characters and
                       the background
                     + "full16" or "default": use the 16 ANSI colours for both the
                       characters and the background (default)
        """
        _lib.caca_set_dither_color.argtypes = [_Dither, ctypes.c_char_p]
        _lib.caca_set_dither_color.restype  = ctypes.c_int

        return _lib.caca_set_dither_color(self, value)

    def get_color(self):
        """ Get current colour mode.
        """
        _lib.caca_get_dither_color.argtypes = [_Dither]
        _lib.caca_get_dither_color.restype  = ctypes.c_char_p

        return _lib.caca_get_dither_color(self)

    def get_color_list(self):
        """ Get available colour modes.
        """
        lst = []

        _lib.caca_get_dither_color_list.argtypes = [_Dither]
        _lib.caca_get_dither_color_list.restype  = ctypes.POINTER(ctypes.c_char_p)

        for item in _lib.caca_get_dither_color_list(self):
            if item is not None and item != "":
                lst.append(item)
            else:
                #memory occurs otherwise
                break

        return lst

    def set_charset(self, value):
        """ Choose characters used for dithering.

            value -- A string describing the characters that need to be
                     used for the dithering.

                     + "ascii" or "default": use only ASCII characters (default).
                     + "shades": use Unicode characters "U+2591 LIGHT SHADE",
                       "U+2592 MEDIUM SHADE" and "U+2593 DARK SHADE". These characters are
                       also present in the CP437 codepage available on DOS and VGA.
                     + "blocks": use Unicode quarter-cell block combinations.
                       These characters are only found in the Unicode set.
        """
        _lib.caca_set_dither_charset.argtypes = [_Dither, ctypes.c_char_p]
        _lib.caca_set_dither_charset.restype  = ctypes.c_int

        return _lib.caca_set_dither_charset(self, value)

    def get_charset(self):
        """ Get current character set.
        """
        _lib.caca_get_dither_charset.argtypes = [_Dither]
        _lib.caca_get_dither_charset.restype  = ctypes.c_char_p

        return _lib.caca_get_dither_charset(self)

    def get_charset_list(self):
        """ Get available dither character sets.
        """
        lst = []

        _lib.caca_get_dither_color_list.argtypes = [_Dither]
        _lib.caca_get_dither_color_list.restype  = ctypes.POINTER(ctypes.c_char_p)

        for item in _lib.caca_get_dither_color_list(self):
            if item is not None and item != "":
                lst.append(item)
            else:
                #memory occurs otherwise
                break

        return lst

    def set_algorithm(self, value):
        """ Set dithering algorithm.

            value -- A string describing the algorithm that needs to be
                     used for the dithering.

                     + "none": no dithering is used, the nearest matching colour is used.
                     + "ordered2": use a 2x2 Bayer matrix for dithering.
                     + "ordered4": use a 4x4 Bayer matrix for dithering.
                     + "ordered8": use a 8x8 Bayer matrix for dithering.
                     + "random": use random dithering.
                     + "fstein": use Floyd-Steinberg dithering (default).
        """
        _lib.caca_set_dither_algorithm.argtypes = [_Dither, ctypes.c_char_p]
        _lib.caca_set_dither_algorithm.restype  = ctypes.c_int

        return _lib.caca_set_dither_algorithm(self, value)

    def get_algorithm(self):
        """ Get dithering algorithms.
        """
        _lib.caca_get_dither_algorithm.argtypes = [_Dither]
        _lib.caca_get_dither_algorithm.restype  = ctypes.c_char_p

        return _lib.caca_get_dither_algorithm(self)

    def get_algorithm_list(self):
        """ Get dithering algorithms.
        """
        lst = []

        _lib.caca_get_dither_color_list.argtypes = [_Dither]
        _lib.caca_get_dither_color_list.restype  = ctypes.POINTER(ctypes.c_char_p)

        for item in _lib.caca_get_dither_color_list(self):
            if item is not None and item != "":
                lst.append(item)
            else:
                #memory occurs otherwise
                break

        return lst

    def bitmap(self, canvas, x, y, width, height, pixels):
        """ Dither a bitmap on the canvas.

            canvas  -- a handle to libcaca canvas
            x       -- X coordinate of the upper-left corner of the drawing area
            y       -- Y coordinate of the upper-left corner of the drawing area
            width   -- width of the drawing area
            height  -- height of the drawing area
            pixels  -- bitmap's pixels
        """
        _lib.caca_dither_bitmap.argtypes = [
            _Canvas, ctypes.c_int, ctypes.c_int,
            ctypes.c_int, ctypes.c_int, _Dither,
            ctypes.c_char_p
        ]
        _lib.caca_dither_bitmap.restype  = ctypes.c_int

        return _lib.caca_dither_bitmap(canvas, x, y, width, height, self, pixels)

class DitherError(Exception):
    pass




# -*- coding: utf-8 -*-
#
# libcaca       Colour ASCII-Art library
#               Python language bindings
# Copyright (c) 2010 Alex Foulon <alxf@lavabit.com>
#               All Rights Reserved
#
# This library is free software. It comes without any warranty, to
# the extent permitted by applicable law. You can redistribute it
# and/or modify it under the terms of the Do What the Fuck You Want
# to Public License, Version 2, as published by Sam Hocevar. See
# http://www.wtfpl.net/ for more details.
#

""" Libcaca Python bindings """

import ctypes

from caca import _lib, _PYTHON3, _bytes_to_str

#color constants
COLOR_BLACK = 0x00
COLOR_BLUE = 0x01
COLOR_GREEN = 0x02
COLOR_CYAN = 0x03
COLOR_RED = 0x04
COLOR_MAGENTA = 0x05
COLOR_BROWN = 0x06
COLOR_LIGHTGRAY = 0x07
COLOR_DARKGRAY = 0x08
COLOR_LIGHTBLUE = 0x09
COLOR_LIGHTGREEN = 0x0a
COLOR_LIGHTCYAN = 0x0b
COLOR_LIGHTRED = 0x0c
COLOR_LIGHTMAGENTA = 0x0d
COLOR_YELLOW = 0x0e
COLOR_WHITE = 0x0f
COLOR_DEFAULT = 0x10
COLOR_TRANSPARENT = 0x20

#styles constants
STYLE_BOLD = 0x01
STYLE_ITALICS = 0x02
STYLE_UNDERLINE = 0x04
STYLE_BLINK = 0x08

#key constants
EVENT_NONE = 0x0000
EVENT_KEY_PRESS = 0x0001
EVENT_KEY_RELEASE = 0x0002
EVENT_MOUSE_PRESS = 0x0004
EVENT_MOUSE_RELEASE = 0x0008
EVENT_MOUSE_MOTION = 0x0010
EVENT_RESIZE = 0x0020
EVENT_QUIT = 0x0040
EVENT_ANY = 0xffff

#event constants
KEY_UNKNOWN = 0x00
KEY_CTRL_A = 0x01
KEY_CTRL_B = 0x02
KEY_CTRL_C = 0x03
KEY_CTRL_D = 0x04
KEY_CTRL_E = 0x05
KEY_CTRL_F = 0x06
KEY_CTRL_G = 0x07
KEY_BACKSPACE = 0x08
KEY_TAB = 0x09
KEY_CTRL_J = 0x0a
KEY_CTRL_K = 0x0b
KEY_CTRL_L = 0x0c
KEY_RETURN = 0x0d
KEY_CTRL_N = 0x0e
KEY_CTRL_O = 0x0f
KEY_CTRL_P = 0x10
KEY_CTRL_Q = 0x11
KEY_CTRL_R = 0x12
KEY_PAUSE = 0x13
KEY_CTRL_T = 0x14
KEY_CTRL_U = 0x15
KEY_CTRL_V = 0x16
KEY_CTRL_W = 0x17
KEY_CTRL_X = 0x18
KEY_CTRL_Y = 0x19
KEY_CTRL_Z = 0x1a
KEY_ESCAPE = 0x1b
KEY_DELETE = 0x7f
KEY_UP = 0x111
KEY_DOWN = 0x112
KEY_LEFT = 0x113
KEY_RIGHT = 0x114
KEY_INSERT = 0x115
KEY_HOME = 0x116
KEY_END = 0x117
KEY_PAGEUP = 0x118
KEY_PAGEDOWN = 0x119
KEY_F1 = 0x11a
KEY_F2 = 0x11b
KEY_F3 = 0x11c
KEY_F4 = 0x11d
KEY_F5 = 0x11e
KEY_F6 = 0x11f
KEY_F7 = 0x120
KEY_F8 = 0x121
KEY_F9 = 0x122
KEY_F10 = 0x123
KEY_F11 = 0x124
KEY_F12 = 0x125
KEY_F13 = 0x126
KEY_F14 = 0x127
KEY_F15 = 0x128


def get_version():
    """ Return string with libcaca version information.
    """
    _lib.caca_get_version.restype = ctypes.c_char_p

    if _PYTHON3:
        return _bytes_to_str(_lib.caca_get_version())
    else:
        return _lib.caca_get_version()

def get_display_driver_list():
    """ Return a list of available drivers as tuple (name, description).
    """
    tmplst = []
    retlst = []

    _lib.caca_get_display_driver_list.restype = ctypes.POINTER(ctypes.c_char_p)

    for item in _lib.caca_get_display_driver_list():
        if item is not None and item != "":
            if _PYTHON3:
                tmplst.append(_bytes_to_str(item))
            else:
                tmplst.append(item)
        else:
            #memory error occured otherwise
            break

    for i in range(0, len(tmplst)):
        if i % 2 == 0:
            retlst.append((tmplst[i], tmplst[i+1]))

    del tmplst
    return retlst

def get_export_list():
    """ Return list of available export formats as tuple (name, description).
    """
    tmplst = []
    retlst = []

    _lib.caca_get_export_list.restype = ctypes.POINTER(ctypes.c_char_p)

    for item in _lib.caca_get_export_list():
        if item is not None and item != "":
            if _PYTHON3:
                tmplst.append(_bytes_to_str(item))
            else:
                tmplst.append(item)
        else:
            #memory error occured otherwise
            break

    for i in range(0, len(tmplst)):
        if i % 2 == 0:
            retlst.append((tmplst[i], tmplst[i+1]))

    del tmplst
    return retlst

def get_import_list():
    """ Return list of available import formats as tuple (name, description).
    """
    tmplst = []
    retlst = []

    _lib.caca_get_import_list.restype = ctypes.POINTER(ctypes.c_char_p)

    autodetect = False
    for item in _lib.caca_get_import_list():
        if item is not None:
            if item == "":
                if not autodetect:
                    if _PYTHON3:
                        tmplst.append(_bytes_to_str("\"\""))
                    else:
                        tmplst.append("\"\"")
                    autodetect = True
                else:
                    #memory error occured otherwise
                    break
            else:
                if _PYTHON3:
                    tmplst.append(_bytes_to_str(item))
                else:
                    tmplst.append(item)
        else:
            #memory error occured otherwise
            break

    for i in range(0, len(tmplst)):
        if i % 2 == 0:
            retlst.append((tmplst[i], tmplst[i+1]))

    del tmplst
    return retlst

def get_font_list():
    """ Return a list of available fonts.
    """
    fl = []

    _lib.caca_get_font_list.restype = ctypes.POINTER(ctypes.c_char_p)

    for item in _lib.caca_get_font_list():
        if item is not None and item != "":
            if _PYTHON3:
                fl.append(_bytes_to_str(item))
            else:
                fl.append(item)
        else:
            #memory error occured otherwise
            break

    return fl

def rand(range_min, range_max):
    """ Generate a random integer within a range.

        range_min   -- the lower bound of the integer range
        range_max   __ the upper bound of the integer range
    """
    _lib.caca_rand.argtypes = [ctypes.c_int, ctypes.c_int]
    _lib.caca_rand.restype  = ctypes.c_int

    return _lib.caca_rand(range_min, range_max)

def attr_to_ansi(attr):
    """ Get DOS ANSI information from attribute.

        attr    -- the requested attribute value
    """
    _lib.caca_attr_to_ansi.argtypes = [ctypes.c_uint32]
    _lib.caca_attr_to_ansi.restype  = ctypes.c_uint8

    return _lib.caca_attr_to_ansi(attr)

def attr_to_ansi_fg(attr):
    """ Get ANSI foreground information from attribute.

        attr    -- the requested attribute value
    """
    _lib.caca_attr_to_ansi_fg.argtypes = [ctypes.c_uint32]
    _lib.caca_attr_to_ansi_fg.restype  = ctypes.c_uint8

    return _lib.caca_attr_to_ansi_fg(attr)

def attr_to_ansi_bg(attr):
    """ Get ANSI background information from attribute.

        attr    -- the requested attribute value
    """
    _lib.caca_attr_to_ansi_bg.argtypes = [ctypes.c_uint32]
    _lib.caca_attr_to_ansi_bg.restype  = ctypes.c_uint8

    return _lib.caca_attr_to_ansi_bg(attr)

def attr_to_rgb12_fg(attr):
    """ Get 12-bit RGB foreground information from attribute.

        attr    -- the requested attribute value
    """
    _lib.caca_attr_to_rgb12_fg.argtypes = [ctypes.c_uint32]
    _lib.caca_attr_to_rgb12_fg.restype  = ctypes.c_uint16

    return _lib.caca_attr_to_rgb12_fg(attr)

def attr_to_rgb12_bg(attr):
    """ Get 12-bit RGB background information from attribute.

        attr    -- the requested attribute value
    """
    _lib.caca_attr_to_rgb12_bg.argtypes = [ctypes.c_uint32]
    _lib.caca_attr_to_rgb12_bg.restype  = ctypes.c_uint16

    return _lib.caca_attr_to_rgb12_bg(attr)

def utf8_to_utf32(ch):
    """ Convert a UTF-8 character to UTF-32.

        ch  -- the character to convert
    """
    _lib.caca_utf8_to_utf32.argtypes = [ctypes.c_char_p,
            ctypes.POINTER(ctypes.c_size_t)
        ]
    _lib.caca_utf8_to_utf32.restype  = ctypes.c_uint32

    return _lib.caca_utf8_to_utf32(ch, ctypes.c_ulong(0))

def utf32_to_utf8(ch):
    """ Convert a UTF-32 character to UTF-8.

        ch  -- the character to convert
    """
    _lib.caca_utf32_to_utf8.argtypes = [ctypes.c_char_p, ctypes.c_uint32]
    _lib.caca_utf32_to_utf8.restype  = ctypes.c_int

    buf = ctypes.c_buffer(7)
    _lib.caca_utf32_to_utf8(buf, ch)

    if _PYTHON3:
        return _bytes_to_str(buf.raw).replace('\x00', '')
    else:
        return buf.raw.replace('\x00', '')

def utf32_to_cp437(ch):
    """ Convert a UTF-32 character to CP437.

        ch  -- the character to convert
    """
    _lib.caca_utf32_to_cp437.argtypes = [ctypes.c_uint32]
    _lib.caca_utf32_to_cp437.restype  = ctypes.c_uint8

    return _lib.caca_utf32_to_cp437(ch)

def cp437_to_utf32(ch):
    """ Convert a CP437 character to UTF-32.

        ch  -- the character to convert
    """
    _lib.caca_cp437_to_utf8.argtypes = [ctypes.c_uint8]
    _lib.caca_cp437_to_utf8.restype  = ctypes.c_uint32

    return _lib.caca_cp437_to_utf8(ch)

def utf32_to_ascii(ch):
    """ Convert a UTF-32 character to ASCII.

        ch  -- the character to convert
    """
    _lib.caca_utf32_to_ascii.argtypes = [ctypes.c_uint32]
    _lib.caca_utf32_to_ascii.restype  = ctypes.c_uint8

    return _lib.caca_utf32_to_ascii(ch)

def utf32_is_fullwidth(ch):
    """ Tell whether a UTF-32 character is fullwidth.

        ch  -- the UTF-32 character
    """
    _lib.caca_utf32_is_fullwidth.argtypes = [ctypes.c_uint32]
    _lib.caca_utf32_is_fullwidth.restype  = ctypes.c_int

    return _lib.caca_utf32_is_fullwidth(ch)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# libcaca       Colour ASCII-Art library
#               Python language bindings
# Copyright (c) 2010 Alex Foulon <alxf@lavabit.com>
#               All Rights Reserved
#
# This library is free software. It comes without any warranty, to
# the extent permitted by applicable law. You can redistribute it
# and/or modify it under the terms of the Do What the Fuck You Want
# to Public License, Version 2, as published by Sam Hocevar. See
# http://www.wtfpl.net/ for more details.
#

""" Libcaca Python bindings """

import caca

if __name__ == '__main__':
    print("libcaca version %s" % caca.get_version())
    print("")
    print("Available drivers:")
    for drv, desc in caca.get_display_driver_list():
        print("  - %s: %s" % (drv, desc))
    print("")
    print("Available fonts:")
    for font in caca.get_font_list():
        print("  - %s" % font)
    print("")
    print("Export formats:")
    for fmt, desc in caca.get_export_list():
        print("  - %s: %s" % (fmt, desc))
    print("")
    print("Import formats:")
    for fmt, desc in caca.get_import_list():
        print("  - %s: %s" % (fmt, desc))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# frames        canvas frame switching features
# Copyright (c) 2010 Alex Foulon <alxf@lavabit.com>
#
# This file is a Python port of "examples/frames.c"
# which is:
# Copyright (c) 2006-2010 Sam Hocevar <sam@hocevar.net>
#               All Rights Reserverd
#
# This library is free software. It comes without any warranty, to
# the extent permitted by applicable law. You can redistribute it
# and/or modify it under the terms of the Do What the Fuck You Want
# to Public License, Version 2, as published by Sam Hocevar. See
# http://www.wtfpl.net/ for more details.
#

import sys

import caca
from caca.canvas import Canvas, CanvasError
from caca.display import Display, DisplayError, Event

def main():
    """ Main function. """

    try:
        cv = Canvas(0, 0)
    except CanvasError as err:
        sys.stderr.write("%s\n" % err)
        sys.exit(2)

    for idx in range(1, 200):
        cv.create_frame(idx)

    sys.stderr.write("canvas created, size is %dx%d\n" \
                        % (cv.get_width(), cv.get_height()))

    cv.set_size(150, 80)
    sys.stderr.write("canvas expanded, size is %dx%d\n" \
                        % (cv.get_width(), cv.get_height()))

    for idx in range(0, 16):
        cv.set_frame(idx)
        cv.set_color_ansi(caca.COLOR_WHITE, idx)
        cv.fill_box(0, 0, 40, 15, ':')
        cv.set_color_ansi(caca.COLOR_WHITE, caca.COLOR_BLUE)
        cv.put_str((idx * 5) // 2, idx, "カカ")
        cv.set_color_ansi(caca.COLOR_DEFAULT, caca.COLOR_TRANSPARENT)

    cv.set_size(41, 16)
    sys.stderr.write("canvas shrinked, size is %dx%d\n" \
                        % (cv.get_width(), cv.get_height()))

    try:
        dp = Display(cv)
    except DisplayError as err:
        sys.stderr.write("%s\n" % err)
        sys.exit(2)

    dp.set_time(50000)
    sys.stderr.write("display attached, size is %dx%d\n" \
                        % (cv.get_width(), cv.get_height()))

    n = 0
    while not dp.get_event(caca.EVENT_KEY_PRESS, Event(), 0):
        cv.set_frame(n % 16)
        dp.refresh()
        n += 1

if __name__ == "__main__":
    main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# text          canvas text import/export
# Copyright (c) 2010 Alex Foulon <alxf@lavabit.com>
#
# This file is a Python port of "examples/text.c"
# which is:
# Copyright (c) 2006-2010 Sam Hocevar <sam@hocevar.net>
#               All Rights Reserverd
#
# This library is free software. It comes without any warranty, to
# the extent permitted by applicable law. You can redistribute it
# and/or modify it under the terms of the Do What the Fuck You Want
# to Public License, Version 2, as published by Sam Hocevar. See
# http://www.wtfpl.net/ for more details.
#

import sys

import caca
from caca.canvas import Canvas, CanvasError, NullCanvas
from caca.display import Display, DisplayError, Event

STRING="""\
              |_| 
   _,----._   | | 
  (/ @  @ \\)   __ 
   |  OO  |   |_ 
   \\ `--' /   |__ 
    `----' 
              |_| 
 Hello world!  | 

"""

def main():
    """ Main function. """

    try:
        pig = Canvas(0, 0)
        pig.import_from_memory(STRING, "text")
        cv = Canvas(pig.get_width() * 2, pig.get_height() * 2)
    except CanvasError as err:
        sys.stderr.write("%s\n" % err)
        sys.exit(2)

    cv.blit(0, 0, pig, NullCanvas())
    pig.flip()
    cv.blit(pig.get_width(), 0, pig, NullCanvas())
    pig.flip()
    pig.flop()
    cv.blit(0, pig.get_height(), pig, NullCanvas())
    pig.flop()
    pig.rotate_180()
    cv.blit(pig.get_width(), pig.get_height(), pig, NullCanvas())

    for j in range(0, cv.get_height()):
        for i in range(0, cv.get_width(), 2):
            cv.set_color_ansi(caca.COLOR_LIGHTBLUE + (i + j) % 6,
                              caca.COLOR_DEFAULT)

            a = cv.get_attr(-1, -1)
            cv.put_attr(i, j, a)
            cv.put_attr(i+1, j, a)

    print("%s" % cv.export_to_memory('utf8'))
    cv.rotate_left()
    print("%s" % cv.export_to_memory('utf8'))

if __name__ == "__main__":
    main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# figfont       libcaca FIGfont test program
# Copyright (c) 2010 Alex Foulon <alxf@lavabit.com>
#
# This file is a Python port of "examples/figfont.c"
# which is:
# Copyright (c) 2007-2010 Sam Hocevar <sam@hocevar.net>
#               All Rights Reserverd
#
# This library is free software. It comes without any warranty, to
# the extent permitted by applicable law. You can redistribute it
# and/or modify it under the terms of the Do What the Fuck You Want
# to Public License, Version 2, as published by Sam Hocevar. See
# http://www.wtfpl.net/ for more details.
#

import codecs
import os
import sys

import caca
from caca.canvas import Canvas, CanvasError

def main():
    """ Main function. """


    if len(sys.argv) < 3:
        sys.stderr.write("Usage: %s <figfont file> <word>\n" \
                            % os.path.basename(sys.argv[0]))
        sys.exit(2)

    try:
        cv = Canvas(0, 0)
    except CanvasError as err:
        sys.stderr.write("%s\n" % err)
        sys.exit(2)

    if cv.set_figfont(sys.argv[1]):
        sys.stderr.write("Could not open font...\n")
        sys.exit(2)

    if sys.version_info[0:2] >= (3,0):
        word = sys.argv[2]
    else:
        word = codecs.decode(sys.argv[2], "utf8")
    for c in word:
        cv.put_figchar(c)

    sys.stderr.write(cv.export_to_memory("utf8"))

if __name__ == "__main__":
    main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# event         event lister for libcaca
# Copyright (c) 2010 Alex Foulon <alxf@lavabit.com>
#
# This file is a Python port of "examples/event.c"
# which is:
# Copyright (c) 2004-2010 Sam Hocevar <sam@hocevar.net>
#               All Rights Reserverd
#
# This library is free software. It comes without any warranty, to
# the extent permitted by applicable law. You can redistribute it
# and/or modify it under the terms of the Do What the Fuck You Want
# to Public License, Version 2, as published by Sam Hocevar. See
# http://www.wtfpl.net/ for more details.
#

import sys

import caca
from caca.canvas import Canvas, CanvasError
from caca.display import Display, DisplayError, Event

def print_event(cv, x, y, event):
    """ Print event name.

        :cv: libcaca canvas
        :x: X coordinate
        :y: Y coordinate
        :event: event to display
    """

    if event.get_type() == caca.EVENT_NONE:
        cv.put_str(x, y, "CACA_EVENT_NONE")
    elif event.get_type() == caca.EVENT_KEY_PRESS:
        character = event.get_key_ch()
        if character > 0x1f and character < 0x80:
            cv.put_str(x, y, "CACA_EVENT_KEY_PRESS 0x%02x (%s)" % (character, chr(character)))
        else:
            cv.put_str(x, y, "CACA_EVENT_KEY_PRESS 0x%02x (%s)" % (character, '?'))
    elif event.get_type() == caca.EVENT_KEY_RELEASE:
        character = event.get_key_ch()
        if character > 0x1f and character < 0x80:
            cv.put_str(x, y, "CACA_EVENT_KEY_RELEASE 0x%02x (%s)" % (character, chr(character)))
        else:
            cv.put_str(x, y, "CACA_EVENT_KEY_RELEASE 0x%02x (%s)" % (character, '?'))
    elif event.get_type() == caca.EVENT_MOUSE_MOTION:
        cv.put_str(x, y, "CACA_EVENT_MOUSE_MOTION %u %u" % (event.get_mouse_x(),
                                                            event.get_mouse_y()))
    elif event.get_type() == caca.EVENT_MOUSE_PRESS:
        cv.put_str(x, y, "CACA_EVENT_MOUSE_PRESS %u" % (event.get_mouse_button()))
    elif event.get_type() == caca.EVENT_MOUSE_RELEASE:
        cv.put_str(x, y, "CACA_EVENT_MOUSE_RELEASE %u" % (event.get_mouse_button()))
    elif event.get_type() == caca.EVENT_RESIZE:
        cv.put_str(x, y, "CACA_EVENT_RESIZE %u %u" % (event.get_resize_width(),
                                                            event.get_resize_height()))
    elif event.get_type() == caca.EVENT_QUIT:
        cv.put_str(x, y, "CACA_EVENT_QUIT")
    else:
        cv.put_str(x, y, "CACA_EVENT_UNKNOWN")

def main():
    """ Main function. """

    events = []
    quit = 0
    quit_string = ["", "q", "qu", "qui", "quit"]

    try:
        cv = Canvas(80, 24)
        dp = Display(cv)
    except (CanvasError, DisplayError) as err:
        sys.stderr.write("%s\n" % err)
        sys.exit(127)

    h = cv.get_height() - 1

    cv.set_color_ansi(caca.COLOR_WHITE, caca.COLOR_BLUE)
    cv.draw_line(0, 0, cv.get_width() - 1, 0, ' ')
    cv.draw_line(0, h, cv.get_width() - 1, h, ' ')
    cv.put_str(0, h, "Type \"quit\" to exit")

    dp.refresh()

    while quit < 4:
        ev = Event()
        if dp.get_event(caca.EVENT_ANY, ev, -1):
            if ev.get_type() == caca.EVENT_KEY_PRESS:
                key = ev.get_key_ch()
                if key == ord('u') and quit == 1:
                    quit += 1
                elif key == ord('i') and quit == 2:
                    quit += 1
                elif key == ord('t') and quit == 3:
                    quit += 1
                elif key == ord('q'):
                    quit = 1
                else:
                    quit = 0

            events.append(ev)

        cv.set_color_ansi(caca.COLOR_LIGHTGRAY, caca.COLOR_BLACK)
        cv.clear()

        #print current event
        cv.set_color_ansi(caca.COLOR_WHITE, caca.COLOR_BLUE)
        cv.draw_line(0, 0, cv.get_width() - 1, 0, ' ')
        if events:
            print_event(cv, 0, 0, events[-1])

        cv.draw_line(0, h, cv.get_width() - 1, h, ' ')
        cv.put_str(0, h, "Type \"quit\" to exit: %s" % quit_string[quit])

        #print previous events
        cv.set_color_ansi(caca.COLOR_WHITE, caca.COLOR_BLACK)
        counts = list(range(0, len(events)-1))
        counts.reverse()
        if len(events) > 1:
            j = 0
            for i in counts:
                if j < h - 1 and events[i].get_type():
                    print_event(cv, 0, ((len(events) - 1) - i), events[i])
                j += 1

        dp.refresh()

if __name__ == "__main__":
    main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# libcaca       Colour ASCII-Art library
#               Python language bindings
# Copyright (c) 2010 Alex Foulon <alxf@lavabit.com>
#               All Rights Reserved
#
# This library is free software. It comes without any warranty, to
# the extent permitted by applicable law. You can redistribute it
# and/or modify it under the terms of the Do What the Fuck You Want
# to Public License, Version 2, as published by Sam Hocevar. See
# http://www.wtfpl.net/ for more details.
#

""" Libcaca Python bindings """

import ctypes
import sys
import time

import caca
from caca.canvas import Canvas
from caca.display import Display, Event

class Drawing(object):
    def __init__(self, canvas):
        self.cv = canvas

    def do_menu(self):
        self.cv.put_str(0, 1, "Drawing Menu")
        self.cv.put_str(0, 2, "------------")
        self.cv.put_str(0, 4, "1) line")
        self.cv.put_str(0, 5, "2) thin line")
        self.cv.put_str(0, 6, "3) polyline")
        self.cv.put_str(0, 7, "4) thin polyline")
        self.cv.put_str(0, 8, "5) circle")
        self.cv.put_str(0, 9, "6) ellipse")
        self.cv.put_str(0, 10, "7) thin ellipse")
        self.cv.put_str(0, 11, "8) box")
        self.cv.put_str(0, 12, "9) thin box")

    def do_line(self, thin=False):
        if thin:
            self.cv.draw_thin_line(0, 0, self.cv.get_width(), 1)
        else:
            self.cv.draw_line(0, 0, self.cv.get_width(), 1, '#')

    def do_polyline(self, thin=False):
        x = [0, self.cv.get_width() - 1, self.cv.get_width() - 1]
        y = [0, self.cv.get_height(), 0]
        array_x = ctypes.c_int * (len(x) + 1)
        array_y = ctypes.c_int * (len(y) + 1)
        array_xy = [ (x, y) for x, y in zip(array_x(*x), array_y(*y))]
        if thin:
            self.cv.draw_thin_polyline(array_xy)
        else:
            self.cv.draw_polyline(array_xy, '#')

    def do_circle(self):
        x = self.cv.get_width() // 2
        y = self.cv.get_height() // 2
        radius = 5
        self.cv.draw_circle(x, y, radius, '@')

    def do_ellipse(self, thin=False):
        x = self.cv.get_width() // 2
        y = self.cv.get_height() // 2
        a = 7
        b = 3
        if thin:
            self.cv.draw_thin_ellipse(x, y, a, b)
        else:
            self.cv.draw_ellipse(x, y, a, b, '@')

    def do_box(self, thin=False):
        if thin:
            self.cv.draw_thin_box(0, 0, self.cv.get_width(), self.cv.get_height())
        else:
            self.cv.draw_box(0, 0, self.cv.get_width(), self.cv.get_height(), '#')

if __name__ == "__main__":
    cv = Canvas()
    dp = Display(cv)
    ev = Event()
    draw = Drawing(cv)

    while True:
        cv.clear()
        draw.do_menu()
        dp.refresh()
        if dp.get_event(caca.EVENT_KEY_PRESS, ev, 0):
            ch = ev.get_key_ch()
            cv.clear()
            if ch == ord('q'):
                sys.exit()
            elif ch == ord('1'):
                draw.do_line()
            elif ch == ord('2'):
                draw.do_line(thin=True)
            elif ch == ord('3'):
                draw.do_polyline()
            elif ch == ord('4'):
                draw.do_polyline(thin=True)
            elif ch == ord('5'):
                draw.do_circle()
            elif ch == ord('6'):
                draw.do_ellipse()
            elif ch == ord('7'):
                draw.do_ellipse(thin=True)
            elif ch == ord('8'):
                draw.do_box()
            elif ch == ord('9'):
                draw.do_box(thin=True)
            dp.refresh()
            time.sleep(2)




# -*- coding: utf-8 -*-
#
# libcaca       Colour ASCII-Art library
#               Python language bindings
# Copyright (c) 2010 Alex Foulon <alxf@lavabit.com>
#               All Rights Reserved
#
# This library is free software. It comes without any warranty, to
# the extent permitted by applicable law. You can redistribute it
# and/or modify it under the terms of the Do What the Fuck You Want
# to Public License, Version 2, as published by Sam Hocevar. See
# http://www.wtfpl.net/ for more details.
#

""" Libcaca Python bindings """

import ctypes

from caca import _lib, _PYTHON3, _str_to_bytes
from caca.canvas import _Canvas, Canvas

class _Display(object):
    """ Model for Display objects.
    """
    def from_param(self):
        """ Required by ctypes module to call object as parameter of
            a C function.
        """
        return self._dp

    def __str__(self):
        return "<CacaDisplay>"

    def __del__(self):
        if self._dp > 0:
            self._free()

    def _free(self):
        """ Free a libcaca display.
        """
        _lib.caca_free_display.argtypes = [_Display]
        _lib.caca_free_display.restype  = ctypes.c_int

        return _lib.caca_free_display(self)

class Display(_Display):
    """ Display objects, methods are libcaca functions with display_t as first
        parameter.
    """
    def __init__(self, cv, driver=None):
        """ Display constructor.

            cv      -- canvas to attach.
            driver  -- caca driver to set with display
        """

        if driver is None:
            _lib.caca_create_display.argtypes = [_Canvas]
            self._dp = _lib.caca_create_display(cv)
        else:
            _lib.caca_create_display_with_driver.argtypes = [
                _Canvas, ctypes.c_char_p
            ]
            if _PYTHON3 and isinstance(driver, str):
                driver = _str_to_bytes(driver)

            self._dp = _lib.caca_create_display_with_driver(cv, driver)

        if self._dp == 0:
            raise DisplayError("Failed to create display")

    def get_driver(self):
        """ Return the caca graphical context's current output driver.
        """
        _lib.caca_get_display_driver.argtypes = [_Display]
        _lib.caca_get_display_driver.restype  = ctypes.c_char_p

        return _lib.caca_get_display_driver(self)

    def set_driver(self, driver=None):
        """ Set the output driver.

            driver  -- A string describing the desired output driver or NULL
                       to choose the best driver automatically.
        """
        _lib.caca_set_display_driver.argtypes = [_Display, ctypes.c_char_p]
        _lib.caca_set_display_driver.restype  = ctypes.c_int

        if not driver:
            driver = ctypes.c_char_p(0)
        else:
            if _PYTHON3 and isinstance(driver, str):
                driver = _str_to_bytes(driver)

        return _lib.caca_set_display_driver(self, driver)

    def get_canvas(self):
        """ Get the canvas attached to a caca graphical context.
        """
        _lib.caca_get_canvas.argtypes = [_Display]
        _lib.caca_get_canvas.restype  = ctypes.POINTER(ctypes.c_char_p)

        return Canvas(pointer=_lib.caca_get_canvas(self))

    def refresh(self):
        """ Flush pending changes and redraw the screen.
        """
        _lib.caca_refresh_display.argtypes = [_Display]
        _lib.caca_refresh_display.restype  = ctypes.c_int

        return _lib.caca_refresh_display(self)

    def set_time(self, usec):
        """ Set the refresh delay.

            usec    -- the refresh delay in microseconds
        """
        _lib.caca_set_display_time.argtypes = [_Display, ctypes.c_int]
        _lib.caca_set_display_time.restype  = ctypes.c_int

        return _lib.caca_set_display_time(self, usec)

    def get_time(self):
        """ Get the display's average rendering time.
        """
        _lib.caca_get_display_time.argtypes = [_Display]
        _lib.caca_get_display_time.restype  = ctypes.c_int

        return _lib.caca_get_display_time(self)

    def set_title(self, title):
        """ Set the display title.

            title   -- the desired display title
        """
        _lib.caca_set_display_title.argtypes = [_Display, ctypes.c_char_p]
        _lib.caca_set_display_title.restype  = ctypes.c_int

        if _PYTHON3 and isinstance(title, str):
            title = _str_to_bytes(title)

        return _lib.caca_set_display_title(self, title)

    def set_mouse(self, flag):
        """ Show or hide the mouse pointer. This function works with the
            ncurses, S-Lang and X11 drivers.

            flag -- 0 hides the pointer, 1 shows the system's default pointer
                    (usually an arrow)
        """
        _lib.caca_set_mouse.argtypes = [_Display, ctypes.c_int]
        _lib.caca_set_mouse.restype  = ctypes.c_int

        return _lib.caca_set_mouse(self, flag)

    def set_cursor(self, flag):
        """ Show or hide the cursor, for devices that support such a feature.

            flag -- 0 hides the cursor, 1 shows the system's default cursor
                    (usually a white rectangle).
        """

        _lib.caca_set_cursor.argtypes = [Display, ctypes.c_int]
        _lib.caca_set_cursor.restype  = ctypes.c_int

        return _lib.caca_set_cursor(self, flag)

    def get_event(self, event_mask, event, timeout):
        """ Get the next mouse or keyboard input event.

            event_mask  -- bitmask of requested events
            event       -- a pointer to caca_event structure or NULL
            tiemout     -- a timeout value in microseconds
        """

        _lib.caca_get_event.argtypes = [
                Display, ctypes.c_int, ctypes.POINTER(Event), ctypes.c_int
            ]

        return _lib.caca_get_event(self, event_mask, ctypes.byref(event),
                                         timeout)

    def get_mouse_x(self):
        """ Return the X mouse coordinate.
        """
        _lib.caca_get_mouse_x.argtypes = [Display]
        _lib.caca_get_mouse_x.restype  = ctypes.c_int

        return _lib.caca_get_mouse_x(self)

    def get_mouse_y(self):
        """ Return the Y mouse coordinate.
        """
        _lib.caca_get_mouse_y.argtypes = [Display]
        _lib.caca_get_mouse_y.restype  = ctypes.c_int

        return _lib.caca_get_mouse_y(self)

class DisplayError(Exception):
    pass

class Event(ctypes.Structure):
    """ Object to store libcaca event.
    """
    _fields_ = (
        ('opaque_structure', ctypes.c_char_p * 32),
    )

    def from_param(self):
        """ Required method to pass object as parameter of a C function.
        """
        return ctypes.byref(self)

    def get_type(self):
        """ Return an event's type.
        """
        _lib.caca_get_event_type.argtypes = [Event]
        _lib.caca_get_event_type.restype  = ctypes.c_int

        return _lib.caca_get_event_type(self)

    def get_key_ch(self):
        """ Return a key press or key release event's value.
        """
        _lib.caca_get_event_key_ch.argtypes = [Event]
        _lib.caca_get_event_key_ch.restype  = ctypes.c_int

        return _lib.caca_get_event_key_ch(self)

    def get_key_utf32(self):
        """ Not implemented.
        """
        raise DisplayError("Not implemented")

    def get_key_utf8(self):
        """ Return a key press or key release event's UTF-8 value
            as python string.
        """
        # set buffer for writing utf8 value
        buf = ctypes.c_buffer(7)

        _lib.caca_get_event_key_utf8.argtypes = [Event, ctypes.c_char_p]
        _lib.caca_get_event_key_utf8.restype  = ctypes.c_int

        _lib.caca_get_event_key_utf8(self, buf)

        raw = []
        for item in list(buf.raw):
            if item == '\x00':
                break
            else:
                raw.append(item)

        return "".join(raw)

    def get_mouse_button(self):
        """ Return a mouse press or mouse release event's button.
        """
        _lib.caca_get_event_mouse_button.argtypes = [Event]
        _lib.caca_get_event_mouse_button.restype  = ctypes.c_int

        return _lib.caca_get_event_mouse_button(self)

    def get_mouse_x(self):
        """ Return a mouse motion event's X coordinate.
        """
        _lib.caca_get_event_mouse_x.argtypes = [Event]
        _lib.caca_get_event_mouse_x.restype  = ctypes.c_int

        return _lib.caca_get_event_mouse_x(self)

    def get_mouse_y(self):
        """ Return a mouse motion event's Y coordinate.
        """
        _lib.caca_get_event_mouse_y.argtypes = [Event]
        _lib.caca_get_event_mouse_y.restype  = ctypes.c_int

        return _lib.caca_get_event_mouse_y(self)

    def get_resize_width(self):
        """ Return a resize event's display width value.
        """
        _lib.caca_get_event_resize_width.argtypes = [Event]
        _lib.caca_get_event_resize_width.restype  = ctypes.c_int

        return _lib.caca_get_event_resize_width(self)

    def get_resize_height(self):
        """ Return a resize event's display height value.
        """
        _lib.caca_get_event_resize_height.argtypes = [Event]
        _lib.caca_get_event_resize_height.restype  = ctypes.c_int

        return _lib.caca_get_event_resize_height(self)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# colors        display all possible libcaca colour pairs
# Copyright (c) 2010 Alex Foulon <alxf@lavabit.com>
#
# This file is a Python port of "examples/colors.c"
# which is:
# Copyright (c) 2003-2010 Sam Hocevar <sam@hocevar.net>
#               All Rights Reserverd
#
# This library is free software. It comes without any warranty, to
# the extent permitted by applicable law. You can redistribute it
# and/or modify it under the terms of the Do What the Fuck You Want
# to Public License, Version 2, as published by Sam Hocevar. See
# http://www.wtfpl.net/ for more details.
#

import sys

import caca
from caca.canvas import Canvas, CanvasError
from caca.display import Display, DisplayError, Event

def main():
    """ Main function. """

    try:
        cv = Canvas(80, 24)
        dp = Display(cv)
    except (CanvasError, DisplayError) as err:
        sys.stderr.write("%s\n" % err)
        sys.exit(127)

    cv.set_color_ansi(caca.COLOR_LIGHTGRAY, caca.COLOR_BLACK)
    cv.clear()

    for i in range(0, 16):
        if i >= 8:
            y = i + 3
        else:
            y = i + 2

        cv.set_color_ansi(caca.COLOR_LIGHTGRAY, caca.COLOR_BLACK)
        cv.printf(3, y, "ANSI %i", i)

        for j in range(0, 16):
            if j >= 8:
                x = 13 + (j * 4)
            else:
                x = 12 + (j * 4)
            if i >= 8:
                y = i + 3
            else:
                y = i + 2

            cv.set_color_ansi(i, j)
            cv.put_str(x, y, "Aaホ")

    cv.set_color_ansi(caca.COLOR_LIGHTGRAY, caca.COLOR_BLACK)
    cv.put_str(3, 20, "This is bold    This is blink    This is italics    This is underline")
    cv.set_attr(caca.STYLE_BOLD)
    cv.put_str(3 + 8, 20, "bold")
    cv.set_attr(caca.STYLE_BLINK)
    cv.put_str(3 + 24, 20, "blink")
    cv.set_attr(caca.STYLE_ITALICS)
    cv.put_str(3 + 41, 20, "italics")
    cv.set_attr(caca.STYLE_UNDERLINE)
    cv.put_str(3 + 60, 20, "underline")

    dp.refresh()
    dp.get_event(caca.EVENT_KEY_PRESS, Event(), -1)

if __name__ == "__main__":
    main()




# -*- coding: utf-8 -*-
#
# libcaca       Colour ASCII-Art library
#               Python language bindings
# Copyright (c) 2010 Alex Foulon <alxf@lavabit.com>
#               All Rights Reserved
#
# This library is free software. It comes without any warranty, to
# the extent permitted by applicable law. You can redistribute it
# and/or modify it under the terms of the Do What the Fuck You Want
# to Public License, Version 2, as published by Sam Hocevar. See
# http://www.wtfpl.net/ for more details.
#

""" Libcaca Python bindings """

#standard modules
import locale
import sys
import ctypes
from ctypes.util import find_library

if find_library('caca') is not None:
    _lib = ctypes.cdll.LoadLibrary(find_library('caca'))
else:
    raise ImportError(
        "Can't find shared library, you need to install libcaca in your path !")

#functions to handle string/bytes in python3+
if sys.version_info[0:2] >= (3, 0):
    _PYTHON3 = True
else:
    _PYTHON3 = False

def _str_to_bytes(the_string):
    """ Translate string to bytes type for python 3.
    """
    return bytes(the_string, locale.getlocale()[1])

def _bytes_to_str(the_bytes):
    """ Translate bytes to string type for python 3.
    """
    return the_bytes.decode(locale.getlocale()[1])

from .common import *




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# figfont       libcaca blit test program
# Copyright (c) 2010 Alex Foulon <alxf@lavabit.com>
#
# This file is a Python port of "examples/blit.c"
# which is:
# Copyright (c) 2006-2010 Sam Hocevar <sam@hocevar.net>
#               All Rights Reserverd
#
# This library is free software. It comes without any warranty, to
# the extent permitted by applicable law. You can redistribute it
# and/or modify it under the terms of the Do What the Fuck You Want
# to Public License, Version 2, as published by Sam Hocevar. See
# http://www.wtfpl.net/ for more details.
#

import sys

import caca
from caca.canvas import Canvas, CanvasError, NullCanvas
from caca.display import Display, DisplayError, Event

THE_PIG="""\
  ,__         __,
  \\)`\\_..._/`(/
   .'  _   _  '.
  /    o\\ /o   \\
  |    .-.-.    |  _
  |   /() ()\\   | (,`)
 / \\  '-----'  / \\ .'
|   '-..___..-'   |
|                 |
|                 |
;                 ;
 \\      / \\      /
  \\-..-/'-'\\-..-/
jgs\\/\\/     \\/\\/"""

def main():
    """ Main function. """
    try:
        cv = Canvas(0, 0)
        dp = Display(cv)
    except (CanvasError, DisplayError) as err:
        sys.stderr.write("%s\n" % err)
        sys.exit(2)

    sprite = Canvas(0, 0)
    sprite.set_color_ansi(caca.COLOR_LIGHTRED, caca.COLOR_BLACK)
    sprite.import_from_memory(THE_PIG, "text")
    sprite.set_handle(sprite.get_width()//2, sprite.get_height()//2)

    cv.set_color_ansi(caca.COLOR_WHITE, caca.COLOR_BLUE)
    cv.put_str(0, 0, "Centered sprite")
    cv.blit(cv.get_width()//2, cv.get_height()//2, sprite, NullCanvas())

    dp.refresh()
    dp.get_event(caca.EVENT_KEY_PRESS, Event(), -1)

    sys.exit(0)

if __name__ == "__main__":
    main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# libcaca       Colour ASCII-Art library
#               Python language bindings
# Copyright (c) 2010 Alex Foulon <alxf@lavabit.com>
#               All Rights Reserved
#
# This library is free software. It comes without any warranty, to
# the extent permitted by applicable law. You can redistribute it
# and/or modify it under the terms of the Do What the Fuck You Want
# to Public License, Version 2, as published by Sam Hocevar. See
# http://www.wtfpl.net/ for more details.
#

""" Libcaca Python bindings """

import random
import sys
import time

import caca

from caca.canvas import Canvas
from caca.display import Display, Event

class CellArray(object):
    def __init__(self, width, height):
        self.array = []
        self.width = width
        self.height = height

        for i in range(0, self.height):
            self.array.append([])
            for j in range(0, self.width):
                self.array[i].append([])

    def get(self, x, y):
        return self.array[x][y]

    def set(self, x, y, value):
        self.array[x][y] = value

    def neighbors(self, x, y):
        n = 0
        h = self.height
        w = self.width

        if self.get((x-1)%h, (y-1)%w):
            n += 1
        if self.get((x-1)%h, y):
            n += 1
        if self.get((x-1)%h, (y+1)%w):
            n += 1
        if self.get(x, (y-1)%w):
            n += 1
        if self.get(x, (y+1)%w):
            n += 1
        if self.get((x+1)%h, (y-1)%w):
            n += 1
        if self.get((x+1)%h, y):
            n += 1
        if self.get((x+1)%h, (y+1)%w):
            n += 1

        return n

    def population(self):
        n = 0

        for i in range(0, self.height):
            for j in range(0, self.width):
                if self.get(i, j):
                    n += 1

        return n

class CellApp(object):
    def __init__(self, width, height):
        self.cycle  = 0
        self.auto   = False
        self.width  = width
        self.height = height

        self.ca   = CellArray(self.width, self.height)
        self.cbuf = CellArray(self.width, self.height)

    def nextCycle(self):
        self.cycle += 1
        for x in range(0, self.ca.height):
            for y in range(0, self.ca.width):
                if self.ca.get(x, y):
                    if self.ca.neighbors(x, y) >= 2 and self.ca.neighbors(x, y) <= 3:
                        self.cbuf.set(x, y, 1)
                    else:
                        self.cbuf.set(x, y, 0)
                elif not self.ca.get(x, y):
                    if self.ca.neighbors(x, y) == 3:
                        self.cbuf.set(x, y, 1)
                    else:
                        self.cbuf.set(x, y, 0)
                else:
                    self.cbuf.set(x, y, 0)
        for x in range(0, self.ca.height):
            for y in range(0, self.ca.width):
                self.ca.set(x, y, self.cbuf.get(x, y))

    def resetCycle(self):
        self.cycle = 0

    def randomCells(self):
        for x in range(0, self.ca.height):
            for y in range(0, self.ca.width):
                self.ca.set(x, y, random.randint(0, 1))

    def renderCells(self, cv):
        cv.clear()
        cv.set_color_ansi(caca.COLOR_WHITE, caca.COLOR_BLUE)
        cv.put_str(0, 0, " "*cv.get_width())
        cv.put_str(0, 0, "s: start, p: pause, n: next, r: random cells, z: reset all")
        cv.put_str(0, cv.get_height()-1, " "*cv.get_width())
        cv.put_str(0, cv.get_height()-1, "generation: %d, population: %d" % (self.cycle, self.ca.population()))
        cv.set_color_ansi(caca.COLOR_DEFAULT, caca.COLOR_BLACK)
        posx = (cv.get_height() - self.height) // 2
        posy = (cv.get_width() - self.width) // 2
        for x in range(0, self.ca.height):
            for y in range(0, self.ca.width):
                if self.ca.get(x, y):
                    cv.put_str(posy+y, posx+x, "@")

    def zeroCells(self):
        for x in range(0, self.ca.height):
            for y in range(0, self.ca.width):
                self.ca.set(x, y, 0)

if __name__ == "__main__":
    cv = Canvas()
    dp = Display(cv)
    ev = Event()

    app = CellApp(80, 20)
    app.zeroCells()

    while True:
        if dp.get_event(caca.EVENT_KEY_PRESS, ev, 2):
            ch = ev.get_key_ch()
            if ch == ord('q'):
                break
            elif ch == ord('s'):
                app.auto = True
            elif ch == ord('n'):
                app.nextCycle()
            elif ch == ord('r'):
                app.randomCells()
            elif ch == ord('p'):
                if app.auto:
                    app.auto = False
                else:
                    app.auto = True
            elif ch == ord('z'):
                app.resetCycle()
                app.zeroCells()
                app.auto = False

        if app.auto:
            app.nextCycle()

        app.renderCells(cv)
        dp.refresh()
        time.sleep(0.2)




#!/usr/bin/env python
#
# Minimal setup.py script
#

import sys
from setuptools import setup

try:
    import caca
except ImportError as err:
    sys.stderr.write("FATAL: %s\n" % str(err))
    sys.exit(127)

version_string=caca.get_version()

setup(
    name='caca',
    author='Alex Foulon',
    author_email='alxf@lavabit.com',
    version=version_string,
    packages=['caca'],
    package_dir={
        'caca': 'caca',
    },
    test_suite='test.alltests',
)




# -*- coding: utf-8 -*-
#
# libcaca       Colour ASCII-Art library
#               Python language bindings
# Copyright (c) 2010 Alex Foulon <alxf@lavabit.com>
#               All Rights Reserved
#
# This library is free software. It comes without any warranty, to
# the extent permitted by applicable law. You can redistribute it
# and/or modify it under the terms of the Do What the Fuck You Want
# to Public License, Version 2, as published by Sam Hocevar. See
# http://www.wtfpl.net/ for more details.
#

""" Libcaca Python bindings """

import ctypes
import errno

from caca import _lib, utf8_to_utf32, utf32_to_utf8
from caca import _PYTHON3, _str_to_bytes, _bytes_to_str
from caca.font import _Font

class _Canvas(object):
    """ Model for Canvas objects.
    """

    def __init__(self):
        self._cv = 0

    def from_param(self):
        """ Required by ctypes module to call object as parameter of
            a C function.
        """
        return self._cv

    def __str__(self):
        return "<CacaCanvas %dx%d>" % (self.get_width(), self.get_height())

    def __del__(self):
        if self._cv > 0:
            self._free()

    def _free(self):
        """ Free a libcaca canvas.
        """
        _lib.caca_free_canvas.argtypes = [_Canvas]
        _lib.caca_free_canvas.restype  = ctypes.c_int

        return _lib.caca_free_canvas(self)

class Canvas(_Canvas):
    """ Canvas object, methods are libcaca functions with canvas_t as
        first parameter.
    """
    def __init__(self, width=0, height=0, pointer=None):
        """ Canvas constructor.

            width   -- the desired canvas width
            height  -- the desired canvas height
            pointer -- pointer to libcaca canvas
        """
        _lib.caca_create_canvas.argtypes = [ctypes.c_int, ctypes.c_int]

        if pointer is None:
            try:
                self._cv = _lib.caca_create_canvas(width, height)
            except ctypes.ArgumentError:
                self._cv = 0
                raise CanvasError("Specified width or height is invalid")
            else:
                if self._cv == 0:
                    err = ctypes.c_int.in_dll(_lib, "errno")
                    if err.value == errno.EINVAL:
                        raise CanvasError("Specified width or height is"
                                          " invalid")
                    elif err.value == errno.ENOMEM:
                        raise CanvasError("Not enough memory for the requested"
                                          " canvas size")
                    else:
                        raise CanvasError("Unknown error: failed to create"
                                          " canvas")
        else:
            self._cv = pointer

    def manage(self, *args, **kw):
        """ Not implemented.
        """
        raise CanvasError("Not implemented")

    def unmanage(self, *args, **kw):
        """ Not implemented.
        """
        raise CanvasError("Not implemented")

    def set_size(self, width, height):
        """ Resize a canvas.

            width   -- the desired canvas width
            height  -- the desired canvas height
        """
        _lib.caca_set_canvas_size.argtypes  = [
                _Canvas, ctypes.c_int, ctypes.c_int
            ]
        _lib.caca_set_canvas_size.restype   = ctypes.c_int

        try:
            ret = _lib.caca_set_canvas_size(self, width, height)
        except ctypes.ArgumentError:
            raise CanvasError("Specified width or height is invalid")
        else:
            if ret == -1:
                err = ctypes.c_int.in_dll(_lib, "errno")
                if err.value == errno.EINVAL:
                    raise CanvasError("Specified width or height is invalid")
                elif err.value == errno.EBUSY:
                    raise CanvasError("The canvas is in use by a display driver"
                                      " and cannot be resized")
                elif err.value == errno.ENOMEM:
                    raise CanvasError("Not enough memory for the requested"
                                      " canvas size")
            else:
                return ret

    def get_width(self):
        """ Get the canvas width.
        """
        _lib.caca_get_canvas_width.argtypes = [_Canvas]
        _lib.caca_get_canvas_width.restype  = ctypes.c_int

        return _lib.caca_get_canvas_width(self)

    def get_height(self):
        """ Get the canvas height.
        """
        _lib.caca_get_canvas_height.argtypes = [_Canvas]
        _lib.caca_get_canvas_height.restype  = ctypes.c_int

        return _lib.caca_get_canvas_height(self)

    def get_chars(self, *args, **kw):
        """ Not implemented.
        """
        raise CanvasError("Not implemented")

    def get_attrs(self, *args, **kw):
        """ Not implemented.
        """
        raise CanvasError("Not implemented")

    def gotoxy(self, x, y):
        """ Set cursor position. Setting the cursor position outside the canvas
            is legal but the cursor will not be shown.

            x   -- X cursor coordinate
            y   -- Y cursor coordinate
        """
        _lib.caca_gotoxy.argtypes = [_Canvas, ctypes.c_int, ctypes.c_int]
        _lib.caca_gotoxy.restyoe  = ctypes.c_int

        try:
            ret = _lib.caca_gotoxy(self, x, y)
        except ctypes.ArgumentError:
            raise CanvasError("specified coordinate X or Y is invalid")
        else:
            return ret

    def wherex(self):
        """ Get X cursor position.
        """
        _lib.caca_wherex.argtypes = [_Canvas]
        _lib.caca_wherex.restype  = ctypes.c_int

        return _lib.caca_wherex(self)

    def wherey(self):
        """ Get Y cursor position.
        """
        _lib.caca_wherey.argtypes = [_Canvas]
        _lib.caca_wherey.restype  = ctypes.c_int

        return _lib.caca_wherey(self)

    def put_char(self, x, y, ch):
        """ Print an ASCII or Unicode character. Return the width of the
            printed character: 2 for a fullwidth character, 1 otherwise.

            x   -- X coordinate
            y   -- Y coordinate
            ch  -- the character to print
        """
        _lib.caca_put_char.argtypes = [
                _Canvas, ctypes.c_int, ctypes.c_int, ctypes.c_uint32
            ]
        _lib.caca_put_char.restype  = ctypes.c_int

        if not isinstance(ch, str):
            raise CanvasError("Specified character is invalid")
        else:
            try:
                ch = ord(ch)
            except TypeError:
                ch = utf8_to_utf32(ch)

            try:
                ret =  _lib.caca_put_char(self, x, y, ch)
            except ctypes.ArgumentError:
                raise CanvasError("specified coordinate X or Y is invalid")
            else:
                return ret

    def get_char(self, x, y):
        """ Get the Unicode character at the given coordinates.

            x   -- X coordinate
            y   -- Y coordinate
        """
        _lib.caca_get_char.argtypes = [
                _Canvas, ctypes.c_int, ctypes.c_int
            ]
        _lib.caca_get_char.restype  = ctypes.c_uint32

        try:
            ch = _lib.caca_get_char(self, x, y)
        except ctypes.ArgumentError:
            raise CanvasError("specified coordinate X or Y is invalid")
        else:
            try:
                ch = ord(ch)
            except TypeError:
                ch = utf32_to_utf8(ch)

            return ch

    def put_str(self, x, y, s):
        """ Print a string.

            x   -- X coordinate
            y   -- Y coordinate
            s   -- the string to print
        """
        _lib.caca_put_str.argtypes = [
                _Canvas, ctypes.c_int, ctypes.c_int, ctypes.c_char_p
            ]
        _lib.caca_put_str.restype  = ctypes.c_int

        if _PYTHON3 and isinstance(s, str):
            s = _str_to_bytes(s)

        try:
            ret = _lib.caca_put_str(self, x, y, s)
        except ctypes.ArgumentError:
            raise CanvasError("Invalid argument")
        else:
            return ret

    def printf(self, x, y, fmt, *args):
        """ Print a formated string.

            x       -- X coordinate
            y       -- Y coordinate
            fmt     -- the format string to print
            args    -- Arguments to the format string
        """
        _lib.caca_printf.argtypes = [
                _Canvas, ctypes.c_int, ctypes.c_int, ctypes.c_char_p
            ]
        _lib.caca_printf.restype  = ctypes.c_int

        if _PYTHON3 and isinstance(fmt, str):
            fmt = _str_to_bytes(fmt)

        if _PYTHON3:
            nargs = []
            for arg in args[:]:
                if isinstance(arg, str):
                    nargs.append(_str_to_bytes(arg))
                else:
                    nargs.append(arg)
        else:
            nargs = args

        try:
            ret = _lib.caca_printf(self, x, y, fmt, *nargs)
        except ctypes.ArgumentError:
            raise CanvasError("Specified coordinate X or Y is invalid")
        else:
            return ret

    def vprintf(self, *args, **kw):
        """ Not implemented.
        """
        raise CanvasError("Not implemented")

    def clear(self):
        """ Clear the canvas.
        """
        _lib.caca_clear_canvas.argtypes = [_Canvas]
        _lib.caca_clear_canvas.restype  = ctypes.c_int

        return _lib.caca_clear_canvas(self)

    def set_handle(self, x, y):
        """ Set cursor handle. Blitting method will use the handle value to
            put the canvas at the proper coordinates.

            x   -- X handle coordinate
            y   -- Y handle coordinate
        """
        _lib.caca_set_canvas_handle.argtypes = [
                _Canvas, ctypes.c_int, ctypes.c_int
            ]
        _lib.caca_set_canvas_handle.restype  = ctypes.c_int

        try:
            ret = _lib.caca_set_canvas_handle(self, x, y)
        except ctypes.ArgumentError:
            raise CanvasError("Specified coordinate X or Y is invalid")
        else:
            return ret

    def get_handle_x(self):
        """ Get X handle position.
        """
        _lib.caca_get_canvas_handle_x.argtypes = [_Canvas]
        _lib.caca_get_canvas_handle_x.restype  = ctypes.c_int

        return _lib.caca_get_canvas_handle_x(self)

    def get_handle_y(self):
        """ Get Y handle position.
        """
        _lib.caca_get_canvas_handle_y.argtypes = [_Canvas]
        _lib.caca_get_canvas_handle_y.restype  = ctypes.c_int

        return _lib.caca_get_canvas_handle_y(self)

    def blit(self, x, y, cv, mask=None):
        """ Blit canvas onto another one.

            x       -- X coordinate
            y       -- Y coordinate
            cv      -- the source canvas
            mask    -- the mask canvas
        """
        _lib.caca_blit.argtypes = [
                _Canvas, ctypes.c_int, ctypes.c_int, _Canvas, _Canvas
            ]
        _lib.caca_blit.restype  = ctypes.c_int

        if not isinstance(cv, Canvas):
            raise CanvasError("Specified mask canvas is invalid")
        else:
            if mask is None:
                mask = NullCanvas()
            else:
                if not isinstance(mask, _Canvas):
                    raise CanvasError("Specified mask canvas is invalid")

        try:
            ret = _lib.caca_blit(self, x, y, cv, mask)
        except ctypes.ArgumentError:
            raise CanvasError("Specified coordinate X or Y is invalid")
        else:
            if ret == -1:
                err = ctypes.c_int.in_dll(_lib, "errno")
                if err.value == errno.EINVAL:
                    raise CanvasError("A mask was specified but the mask size"
                                      " and source canvas size do not match")
            else:
                return ret

    def set_boundaries(self, x, y, width, height):
        """ Set a canvas' new boundaries.

            x       -- X coordinate of the top-left corner
            y       -- Y coordinate of the top-left corner
            width   -- width of the box
            height  -- height of the box
        """
        _lib.caca_set_canvas_boundaries.argtypes = [
              _Canvas, ctypes.c_int, ctypes.c_int, ctypes.c_int, ctypes.c_int
            ]
        _lib.caca_set_canvas_boundaries.restype  = ctypes.c_int

        try:
            ret = _lib.caca_set_canvas_boundaries(self, x, y, width, height)
        except ctypes.ArgumentError:
            raise CanvasError("Specified coordinate or size is invalid")
        else:
            if ret == -1:
                err = ctypes.c_int.in_dll(_lib, "errno")
                if err.value == errno.EINVAL:
                    raise CanvasError("Specified width or height is invalid")
                elif err.value == errno.EBUSY:
                    raise CanvasError("The canvas is in use by a display driver"
                                      " and cannot be resized")
                elif err.value == errno.ENOMEM:
                    raise CanvasError("Not enough memory for the requested"
                                      " canvas size")
            else:
                return ret

    def disable_dirty_rect(self):
        """ Disable dirty rectangles.
        """
        _lib.caca_disable_dirty_rect.argtypes = [_Canvas]
        _lib.caca_disable_dirty_rect.restype  = ctypes.c_int

        return _lib.caca_disable_dirty_rect(self)

    def enable_dirty_rect(self):
        """ Enable dirty rectangles.
        """
        _lib.caca_enable_dirty_rect.argtypes = [_Canvas]
        _lib.caca_enable_dirty_rect.restype  = ctypes.c_int

        ret = _lib.caca_enable_dirty_rect(self)
        if ret == -1:
            err = ctypes.c_int.in_dll(_lib, "errno")
            if err.value == errno.EINVAL:
                raise CanvasError("Dirty rectangles were not disabled")
        else:
            return ret

    def get_dirty_rect_count(self):
        """ Get the number of dirty rectangles in the canvas.
        """
        _lib.caca_get_dirty_rect_count.argtypes = [_Canvas]
        _lib.caca_get_dirty_rect_count.restype  = ctypes.c_int

        return _lib.caca_get_dirty_rect_count(self)

    def get_dirty_rect(self, idx):
        """ Get a canvas's dirty rectangle. Return python dictionnary with
            coords as keys: x, y, width, height.

            idx -- the requested rectangle index
        """
        dct = None
        x = ctypes.c_int()
        y = ctypes.c_int()
        width  = ctypes.c_int()
        height = ctypes.c_int()

        _lib.caca_get_dirty_rect.argtypes = [
                _Canvas, ctypes.c_int,
                ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int),
                ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int)
            ]
        _lib.caca_get_dirty_rect.restype  = ctypes.c_int

        try:
            ret = _lib.caca_get_dirty_rect(self, idx, x, y, width, height)
        except ctypes.ArgumentError:
            raise CanvasError("Specified rectangle index is invalid")
        else:
            if ret == -1:
                err = ctypes.c_int.in_dll(_lib, "errno")
                if err.value == errno.EINVAL:
                    raise CanvasError("Specified rectangle index is out of"
                                      " bounds")
            else:
                dct = {
                    'x': x.value, 'y': y.value,
                    'width': width.value, 'height': height.value,
                }
                return dct

    def add_dirty_rect(self, x, y, width, height):
        """ Add an area to the canvas's dirty rectangle list.

            x       -- the leftmost edge of the additional dirty rectangle
            y       -- the topmost edge of the additional dirty rectangle
            width   -- the width of the additional dirty rectangle
            height  -- the height of the additional dirty rectangle
        """
        _lib.caca_add_dirty_rect.argtypes = [
                _Canvas, ctypes.c_int, ctypes.c_int,
                ctypes.c_int, ctypes.c_int
            ]
        _lib.caca_add_dirty_rect.restype  = ctypes.c_int

        try:
            ret =_lib.caca_add_dirty_rect(self, x, y, width, height)
        except ctypes.ArgumentError:
            raise CanvasError("Specified coordinate or size is invalid")
        else:
            if ret == -1:
                err = ctypes.c_int.in_dll(_lib, "errno")
                if err.value == errno.EINVAL:
                    raise CanvasError("Specified rectangle coordinates are out"
                                      " of bounds")
            else:
                return ret

    def remove_dirty_rect(self, x, y, width, height):
        """ Remove an area from the dirty rectangle list.

            x       -- the leftmost edge of the additional dirty rectangle
            y       -- the topmost edge of the additional dirty rectangle
            width   -- the width of the additional rectangle
            height  -- the height of the additional dirty rectangle
        """
        _lib.caca_remove_dirty_rect.argtypes = [
                _Canvas, ctypes.c_int, ctypes.c_int,
                ctypes.c_int, ctypes.c_int
            ]
        _lib.caca_remove_dirty_rect.restype  = ctypes.c_int

        try:
            ret = _lib.caca_remove_dirty_rect(self, x, y, width, height)
        except ctypes.ArgumentError:
            raise CanvasError("Specified coordinate or size is invalid")
        else:
            if ret == -1:
                err = ctypes.c_int.in_dll(_lib, "errno")
                if err.value == errno.EINVAL:
                    raise CanvasError("Specified rectangle coordinates are out"
                                      " of bounds")
            else:
                return ret

    def clear_dirty_rect_list(self):
        """ Clear a canvas's dirty rectangle list.
        """
        _lib.caca_clear_dirty_rect_list.argtypes = [_Canvas]
        _lib.caca_clear_dirty_rect_list.restype  = ctypes.c_int

        return _lib.caca_clear_dirty_rect_list(self)

    def invert(self):
        """ Invert a canvas' colours.
        """
        _lib.caca_invert.argtypes = [_Canvas]
        _lib.caca_invert.restype  = ctypes.c_int

        return _lib.caca_invert(self)

    def flip(self):
        """ Flip a canvas horizontally.
        """
        _lib.caca_flip.argtypes = [_Canvas]
        _lib.caca_flip.restype  = ctypes.c_int

        return _lib.caca_flip(self)

    def flop(self):
        """ Flip a canvas vertically.
        """
        _lib.caca_flop.argtypes = [_Canvas]
        _lib.caca_flop.restype  = ctypes.c_int

        return _lib.caca_flop(self)

    def rotate_180(self):
        """ Rotate a canvas.
        """
        _lib.caca_rotate_180.argtypes = [_Canvas]
        _lib.caca_rotate_180.restype  = ctypes.c_int

        return _lib.caca_rotate_180(self)

    def rotate_left(self):
        """ Rotate a canvas, 90 degrees counterclockwise.
        """
        _lib.caca_rotate_left.argtypes = [_Canvas]
        _lib.caca_rotate_left.restype  = ctypes.c_int

        ret = _lib.caca_rotate_left(self)
        if ret == -1:
            err = ctypes.c_int.in_dll(_lib, "errno")
            if err.value == errno.EBUSY:
                raise CanvasError("The canvas is in use by a display driver"
                                  " and cannot be rotated")
            elif err.value == errno.ENOMEM:
                raise CanvasError("Not enough memory to allocate the new"
                                  " canvas size")
        else:
            return ret

    def rotate_right(self):
        """ Rotate a canvas, 90 degrees clockwise.
        """
        _lib.caca_rotate_right.argtypes = [_Canvas]
        _lib.caca_rotate_right.restype  = ctypes.c_int

        ret = _lib.caca_rotate_right(self)
        if ret == -1:
            err = ctypes.c_int.in_dll(_lib, "errno")
            if err.value == errno.EBUSY:
                raise CanvasError("The canvas is in use by a display driver"
                                  " and cannot be rotated")
            elif err.value == errno.ENOMEM:
                raise CanvasError("Not enough memory to allocate the new"
                                  " canvas size")
        else:
            return ret

    def stretch_left(self):
        """ Rotate and stretch a canvas, 90 degrees counterclockwise.
        """
        _lib.caca_stretch_left.argtypes = [_Canvas]
        _lib.caca_stretch_left.restype  = ctypes.c_int

        ret = _lib.caca_stretch_left(self)
        if ret == -1:
            err = ctypes.c_int.in_dll(_lib, "errno")
            if err.value == errno.EBUSY:
                raise CanvasError("The canvas is in use by a display driver"
                                  " and cannot be rotated")
            elif err.value == errno.ENOMEM:
                raise CanvasError("Not enough memory to allocate the new"
                                  " canvas size")
        else:
            return ret

    def stretch_right(self):
        """ Rotate and stretch a canvas, 90 degrees clockwise.
        """
        _lib.caca_stretch_right.argtypes = [_Canvas]
        _lib.caca_stretch_right.restype  = ctypes.c_int

        ret = _lib.caca_stretch_right(self)
        if ret == -1:
            err = ctypes.c_int.in_dll(_lib, "errno")
            if err.value == errno.EBUSY:
                raise CanvasError("The canvas is in use by a display driver"
                                  " and cannot be rotated")
            elif err.value == errno.ENOMEM:
                raise CanvasError("Not enough memory to allocate the new"
                                  " canvas size")
        else:
            return ret

    def get_attr(self, x, y):
        """ Get the text attribute at the given coordinates.

            x   -- X coordinate
            y   -- Y coordinate
        """
        _lib.caca_get_attr.argtypes = [_Canvas, ctypes.c_int, ctypes.c_int]
        _lib.caca_get_attr.restype  = ctypes.c_uint32

        try:
            ret = _lib.caca_get_attr(self, x, y)
        except ctypes.ArgumentError:
            raise CanvasError("Specified coordinate X or Y is invalid")
        else:
            return ret

    def set_attr(self, attr):
        """ Set the default character attribute.

            attr    -- the requested attribute value
        """
        _lib.caca_set_attr.argtypes = [_Canvas, ctypes.c_uint32]
        _lib.caca_set_attr.restype  = ctypes.c_int

        try:
            ret = _lib.caca_set_attr(self, attr)
        except ctypes.ArgumentError:
            raise CanvasError("Specified attribute is invalid")
        else:
            return ret

    def unset_attr(self, attr):
        """ Unset the default character attribute.

            attr    -- the requested attribute value
        """
        _lib.caca_unset_attr.argtypes = [_Canvas, ctypes.c_uint32]
        _lib.caca_unset_attr.restype  = ctypes.c_int

        try:
            ret = _lib.caca_unset_attr(self, attr)
        except ctypes.ArgumentError:
            raise CanvasError("Specified attribute is invalid")
        else:
            return ret

    def toggle_attr(self, attr):
        """ Toggle the default character attribute.

            attr -- the requested attribute value
        """
        _lib.caca_toggle_attr.argtypes = [_Canvas, ctypes.c_uint32]
        _lib.caca_toggle_attr.restype  = ctypes.c_int

        try:
            ret = _lib.caca_toggle_attr(self, attr)
        except ctypes.ArgumentError:
            raise CanvasError("Specified attribute is invalid")
        else:
            return ret

    def put_attr(self, x, y, attr):
        """ Set the character attribute at the given coordinates.

            x       -- X coordinate
            y       -- Y coordinate
            attr    -- the requested attribute value
        """
        _lib.caca_put_attr.argtypes = [
            _Canvas, ctypes.c_int, ctypes.c_int, ctypes.c_uint32
        ]
        _lib.caca_put_attr.restype  = ctypes.c_int

        try:
            ret = _lib.caca_put_attr(self, x, y, attr)
        except ctypes.ArgumentError:
            raise CanvasError("Specified coordinate or attribute is invalid")
        else:
            return ret

    def set_color_ansi(self, fg, bg):
        """ Set the default colour pair for text (ANSI version).

            fg  -- the requested ANSI foreground colour.
            bg  -- the requested ANSI background colour.
        """
        _lib.caca_set_color_ansi.argtypes = [
                _Canvas, ctypes.c_uint8, ctypes.c_uint8
            ]
        _lib.caca_set_color_ansi.restype  = ctypes.c_int

        try:
            ret = _lib.caca_set_color_ansi(self, fg, bg)
        except ctypes.ArgumentError:
            raise CanvasError("At least one of the colour values is invalid")
        else:
            if ret == -1:
                err = ctypes.c_int.in_dll(_lib, "errno")
                if err.value == errno.EINVAL:
                    raise CanvasError("At least one of the colour values"
                                      " is invalid")
            else:
                return ret

    def set_color_argb(self, fg, bg):
        """ Set the default colour pair for text (truecolor version).

            fg  -- the requested ARGB foreground colour.
            bg  -- the requested ARGB background colour.
        """
        _lib.caca_set_color_argb.argtypes = [
            _Canvas, ctypes.c_uint16, ctypes.c_uint16
        ]
        _lib.caca_set_color_argb.restype  = ctypes.c_int

        try:
            ret = _lib.caca_set_color_argb(self, fg, bg)
        except ctypes.ArgumentError:
            raise CanvasError("At least one of the colour values is invalid")
        else:
            return ret

    def draw_line(self, x1, y1, x2, y2, ch):
        """ Draw a line on the canvas using the given character.

            x1  -- X coordinate of the first point
            y1  -- Y coordinate of the first point
            x2  -- X coordinate of the second point
            y2  -- Y coordinate of the second point
            ch  -- character to be used to draw the line
        """
        _lib.caca_draw_line.argtypes = [
                _Canvas, ctypes.c_int, ctypes.c_int,
                ctypes.c_int, ctypes.c_int, ctypes.c_uint32
            ]
        _lib.caca_draw_line.restype  = ctypes.c_int

        if not isinstance(ch, str):
            raise CanvasError("Specified character is invalid")
        else:
            try:
                ch = ord(ch)
            except TypeError:
                ch = utf8_to_utf32(ch)

            try:
                ret = _lib.caca_draw_line(self, x1, y1, x2, y2, ch)
            except ctypes.ArgumentError:
                raise CanvasError("specified coordinate is invalid")
            else:
                return ret

    def draw_polyline(self, array_xy, ch):
        """ Draw a polyline.

            array_xy -- List of (X, Y) coordinates
            ch       -- character to be used to draw the line
        """
        if not isinstance(array_xy, list) or len(array_xy) < 2:
            raise CanvasError("Specified array of coordinates is invalid")
        else:
            for item in array_xy:
                if not isinstance(item, list) and \
                   not isinstance(item, tuple):
                    raise CanvasError("Specified array of coordinates"
                                      " is invalid")

        ax = ctypes.c_int * len(array_xy)
        ay = ctypes.c_int * len(array_xy)

        _lib.caca_draw_polyline.argtypes = [
                _Canvas, ax, ay, ctypes.c_int, ctypes.c_uint32
            ]
        _lib.caca_draw_polyline.restype  = ctypes.c_int

        if not isinstance(ch, str):
            raise CanvasError("Specified character is invalid")
        else:
            try:
                ch = ord(ch)
            except TypeError:
                ch = utf8_to_utf32(ch)

            try:
                ax = ax(*[x[0] for x in array_xy])
                ay = ay(*[y[1] for y in array_xy])
            except IndexError:
                raise CanvasError("Specified array coordinates is invalid")

            try:
                ret = _lib.caca_draw_polyline(self, ax, ay,
                                              len(array_xy) - 1, ch)
            except ctypes.ArgumentError:
                raise CanvasError("specified array of coordinates is invalid")
            else:
                return ret

    def draw_thin_line(self, x1, y1, x2, y2):
        """ Draw a thin line on the canvas, using ASCII art.

            x1  -- X coordinate of the first point
            y1  -- Y coordinate of the first point
            x2  -- X coordinate of the second point
            y2  -- Y coordinate of the second point
        """
        _lib.caca_draw_thin_line.argtypes = [
                _Canvas, ctypes.c_int, ctypes.c_int,
                ctypes.c_int, ctypes.c_int
            ]
        _lib.caca_draw_thin_line.restype  = ctypes.c_int

        try:
            ret = _lib.caca_draw_thin_line(self, x1, y1, x2, y2)
        except ctypes.ArgumentError:
            raise CanvasError("specified coordinate is invalid")
        else:
            return ret

    def draw_thin_polyline(self, array_xy):
        """ Draw an ASCII art thin polyline.

            array_xy -- Array of (X, Y) coordinates
        """
        if not isinstance(array_xy, list) or len(array_xy) < 2:
            raise CanvasError("Specified array of coordinates is invalid")
        else:
            for item in array_xy:
                if not isinstance(item, list) and \
                   not isinstance(item, tuple):
                    raise CanvasError("Specified array of coordinates"
                                      " is invalid")

        ax = ctypes.c_int * len(array_xy)
        ay = ctypes.c_int * len(array_xy)

        _lib.caca_draw_thin_polyline.argtypes = [
               Canvas, ax, ay, ctypes.c_int
            ]
        _lib.caca_draw_thin_polyline.restype  = ctypes.c_int

        try:
            ax = ax(*[x[0] for x in array_xy])
            ay = ay(*[y[1] for y in array_xy])
        except IndexError:
            raise CanvasError("Specified array coordinates is invalid")

        try:
            ret = _lib.caca_draw_thin_polyline(self, ax, ay, len(array_xy) - 1)
        except ctypes.ArgumentError:
            raise CanvasError("specified array of coordinates is invalid")
        else:
            return ret

    def draw_circle(self, x, y, r, ch):
        """ Draw a circle on the canvas using the given character.

            x   -- center X coordinate
            y   -- center Y coordinate
            r   -- circle radius
            ch  -- the UTF-32 character to be used to draw the circle outline
        """
        _lib.caca_draw_circle.argtypes = [
            _Canvas, ctypes.c_int, ctypes.c_int, ctypes.c_int, ctypes.c_uint32
        ]
        _lib.caca_draw_circle.restype  = ctypes.c_int

        if not isinstance(ch, str):
            raise CanvasError("Specified character is invalid")
        else:
            try:
                ch = ord(ch)
            except TypeError:
                ch = utf8_to_utf32(ch)

        try:
            ret = _lib.caca_draw_circle(self, x, y, r, ch)
        except ctypes.ArgumentError:
            raise CanvasError("Specified circle coordinate or radius is"
                              " invalid")
        else:
            return ret

    def draw_ellipse(self, xo, yo, a, b, ch):
        """ Draw an ellipse on the canvas using the given character.

            xo  -- center X coordinate
            yo  -- center Y coordinate
            a   -- ellipse x radius
            b   -- ellipse y radius
            ch  -- UTF-32 character to be used to draw the ellipse outline
        """
        _lib.caca_draw_ellipse.argtypes = [
                _Canvas, ctypes.c_int, ctypes.c_int,
                ctypes.c_int, ctypes.c_int, ctypes.c_uint32
            ]
        _lib.caca_draw_ellipse.restype  = ctypes.c_int

        if not isinstance(ch, str):
            raise CanvasError("Specified character is invalid")
        else:
            try:
                ch = ord(ch)
            except TypeError:
                ch = utf8_to_utf32(ch)

        try:
            ret = _lib.caca_draw_ellipse(self, xo, yo, a, b, ch)
        except ctypes.ArgumentError:
            raise CanvasError("Specified ellipse coordinate or radius is"
                              " invalid")
        else:
            return ret

    def draw_thin_ellipse(self, xo, yo, a, b):
        """ Draw a thin ellipse on the canvas.

            xo  -- center X coordinate
            yo  -- center Y coordinate
            a   -- ellipse X radius
            b   -- ellipse Y radius
        """
        _lib.caca_draw_thin_ellipse.argtypes = [
                _Canvas, ctypes.c_int, ctypes.c_int,
                ctypes.c_int, ctypes.c_int
            ]
        _lib.caca_draw_thin_ellipse.restype  = ctypes.c_int

        try:
            ret = _lib.caca_draw_thin_ellipse(self, xo, yo, a, b)
        except ctypes.ArgumentError:
            raise CanvasError("Specified ellipse coordinate or radius is"
                              " invalid")
        else:
            return ret

    def fill_ellipse(self, xo, yo, a, b, ch):
        """ Fill an ellipse on the canvas using the given character.

            xo  -- center X coordinate
            yo  -- center Y coordinate
            a   -- ellipse X radius
            b   -- ellipse Y radius
            ch  -- UTF-32 character to be used to fill the ellipse
        """
        _lib.caca_fill_ellipse.argtypes = [
                _Canvas, ctypes.c_int, ctypes.c_int,
                ctypes.c_int, ctypes.c_int, ctypes.c_uint32
            ]
        _lib.caca_fill_ellipse.restype  = ctypes.c_int

        if not isinstance(ch, str):
            raise CanvasError("Specified character is invalid")
        else:
            try:
                ch = ord(ch)
            except TypeError:
                ch = utf8_to_utf32(ch)

        try:
            ret = _lib.caca_fill_ellipse(self, xo, yo, a, b, ch)
        except ctypes.ArgumentError:
            raise CanvasError("Specified ellipse coordinate or radius is"
                              " invalid")
        else:
            return ret

    def draw_box(self, x, y, width, height, ch):
        """ Draw a box on the canvas using the given character.

            x       -- X coordinate of the upper-left corner of the box
            y       -- Y coordinate of the upper-left corner of the box
            width   -- width of the box
            height  -- height of the box
            ch      -- character to be used to draw the box
        """
        _lib.caca_draw_box.argtypes = [
                Canvas, ctypes.c_int, ctypes.c_int,
                ctypes.c_int, ctypes.c_int, ctypes.c_uint32
            ]
        _lib.caca_draw_box.restype  = ctypes.c_int

        if not isinstance(ch, str):
            raise CanvasError("Specified character is invalid")
        else:
            try:
                ch = ord(ch)
            except TypeError:
                ch = utf8_to_utf32(ch)

        try:
            ret = _lib.caca_draw_box(self, x, y, width, height, ch)
        except ctypes.ArgumentError:
            raise CanvasError("specified box coordinate is invalid")
        else:
            return ret

    def draw_thin_box(self, x, y, width, height):
        """ Draw a thin box on the canvas.

            x       -- X coordinate of the upper-left corner of the box
            y       -- Y coordinate of the upper-left corner of the box
            width   -- width of the box
            height  -- height of the box
        """
        _lib.caca_draw_thin_box.argtypes = [
                _Canvas, ctypes.c_int, ctypes.c_int,
                ctypes.c_int, ctypes.c_int
            ]
        _lib.caca_draw_thin_box.restype  = ctypes.c_int

        try:
            ret = _lib.caca_draw_thin_box(self, x, y, width, height)
        except ctypes.ArgumentError:
            raise CanvasError("specified box coordinate is invalid")
        else:
            return ret

    def draw_cp437_box(self, x, y, width, height):
        """ Draw a box on the canvas using CP437 characters.

            x       -- X coordinate of the upper-left corner box
            y       -- Y coordinate of the upper-left corner box
            width   -- width of the box
            height  -- height of the box
        """
        _lib.caca_draw_cp437_box.argtypes = [
                _Canvas, ctypes.c_int, ctypes.c_int,
                ctypes.c_int, ctypes.c_int
            ]
        _lib.caca_draw_cp437_box.restype  = ctypes.c_int

        try:
            ret = _lib.caca_draw_cp437_box(self, x, y, width, height)
        except ctypes.ArgumentError:
            raise CanvasError("specified box coordinate is invalid")
        else:
            return ret

    def fill_box(self, x, y, width, height, ch):
        """ Fill a box on the canvas using the given character.

            x       -- X coordinate of the upper-left corner of the box
            y       -- Y coordinate of the upper-left corner of the box
            width   -- width of the box
            height  -- height of the box
            ch      -- UFT-32 character to be used to fill the box
        """
        _lib.caca_fill_box.argtypes = [
            _Canvas, ctypes.c_int, ctypes.c_int,
            ctypes.c_int, ctypes.c_int, ctypes.c_uint32
        ]
        _lib.caca_fill_box.restype  = ctypes.c_int

        if not isinstance(ch, str):
            raise CanvasError("Specified character is invalid")
        else:
            try:
                ch = ord(ch)
            except TypeError:
                ch = utf8_to_utf32(ch)

        try:
            ret = _lib.caca_fill_box(self, x, y, width, height, ch)
        except ctypes.ArgumentError:
            raise CanvasError("specified box coordinate is invalid")
        else:
            return ret

    def draw_triangle(self, x1, y1, x2, y2, x3, y3, ch):
        """ Draw a triangle on the canvas using the given character.

            x1  -- X coordinate of the first point
            y1  -- Y coordinate of the first point
            x2  -- X coordinate of the second point
            y2  -- Y coordinate of the second point
            x3  -- X coordinate of the third point
            y3  -- Y coordinate of the third point
            ch  -- UTF-32 character to be used to draw the triangle outline
        """
        _lib.caca_draw_triangle.argtypes = [
            _Canvas, ctypes.c_int, ctypes.c_int, ctypes.c_int,
            ctypes.c_int, ctypes.c_int, ctypes.c_int, ctypes.c_uint32
        ]
        _lib.caca_draw_triangle.restype  = ctypes.c_int

        if not isinstance(ch, str):
            raise CanvasError("Specified character is invalid")
        else:
            try:
                ch = ord(ch)
            except TypeError:
                ch = utf8_to_utf32(ch)

        try:
            ret = _lib.caca_draw_triangle(self, x1, y1, x2, y2, x3, y3, ch)
        except ctypes.ArgumentError:
            raise CanvasError("specified triangle coordinate is invalid")
        else:
            return ret

    def draw_thin_triangle(self, x1, y1, x2, y2, x3, y3):
        """ Draw a thin triangle on the canvas.

            x1  -- X coordinate of the first point
            y1  -- Y coordinate of the first point
            x2  -- X coordinate of the second point
            y2  -- Y coordinate of the second point
            x3  -- X coordinate of the third point
            y3  -- Y coordinate of the third point
        """
        _lib.caca_draw_thin_triangle.argtypes = [
            _Canvas, ctypes.c_int, ctypes.c_int, ctypes.c_int,
            ctypes.c_int, ctypes.c_int, ctypes.c_int
        ]
        _lib.caca_draw_thin_triangle.restype  = ctypes.c_int

        try:
            ret = _lib.caca_draw_thin_triangle(self, x1, y1, x2, y2, x3, y3)
        except ctypes.ArgumentError:
            raise CanvasError("specified triangle coordinate is invalid")
        else:
            return ret

    def fill_triangle(self, x1, y1, x2, y2, x3, y3, ch):
        """ Fill a triangle on the canvas using the given character.

            x1  -- X coordinate of the first point
            y1  -- Y coordinate of the first point
            x2  -- X coordinate of the second point
            y2  -- Y coordinate of the second point
            x3  -- X coordinate of the second point
            y3  -- Y coordinate of the second point
            ch  -- UTF-32 character to be used to fill the triangle
        """
        _lib.caca_fill_triangle.argtypes = [
            _Canvas, ctypes.c_int, ctypes.c_int, ctypes.c_int,
            ctypes.c_int, ctypes.c_int, ctypes.c_int
        ]
        _lib.caca_fill_triangle.restype  = ctypes.c_int

        if not isinstance(ch, str):
            raise CanvasError("Specified character is invalid")
        else:
            try:
                ch = ord(ch)
            except TypeError:
                ch = utf8_to_utf32(ch)

        try:
            ret = _lib.caca_fill_triangle(self, x1, y1, x2, y2, x3, y3, ch)
        except ctypes.ArgumentError:
            raise CanvasError("specified triangle coordinate is invalid")
        else:
            return ret

    def fill_triangle_textured(self, coords, tex, uv):
        """ Fill a triangle on the canvas using an arbitrary-sized texture.

            coords  -- coordinates of the triangle (3{x,y})
            tex     -- the handle of the canvas texture
            uv      -- coordinates of the texture  (3{u,v})
        """
        _lib.caca_fill_triangle_textured.argtypes = [
            _Canvas, ctypes.c_int * 6, _Canvas, ctypes.c_int * 6
        ]
        _lib.caca_fill_triangle_textured.restype  = ctypes.c_int

        return _lib.caca_fill_triangle_textured(self, coords, tex, uv)

    def get_frame_count(self):
        """ Get the number of frames in a canvas.
        """
        _lib.caca_get_frame_count.argtypes = [_Canvas]
        _lib.caca_get_frame_count.restype  = ctypes.c_int

        return _lib.caca_get_frame_count(self)

    def set_frame(self, idx):
        """ Activate a given canvas frame.

            idx -- the canvas frame to activate
        """
        _lib.caca_set_frame.argtypes = [_Canvas, ctypes.c_int]
        _lib.caca_set_frame.restype  = ctypes.c_int

        try:
            ret = _lib.caca_set_frame(self, idx)
        except ctypes.ArgumentError:
            raise CanvasError("specified index is invalid")
        else:
            err = ctypes.c_int.in_dll(_lib, "errno")
            if err.value == errno.EINVAL:
                raise CanvasError("Requested frame is out of range")
            else:
                return ret

    def get_frame_name(self):
        """ Get the current frame's name.
        """
        _lib.caca_get_frame_name.argtypes = [_Canvas]
        _lib.caca_get_frame_name.restype  = ctypes.c_char_p

        if _PYTHON3:
            return _bytes_to_str(_lib.caca_get_frame_name(self))
        else:
            return _lib.caca_get_frame_name(self)

    def set_frame_name(self, name):
        """ Set the current frame's name.

            name    -- the name to give to the current frame
        """
        _lib.caca_set_frame_name.argtypes = [_Canvas, ctypes.c_char_p]
        _lib.caca_set_frame_name.restype  = ctypes.c_int

        if _PYTHON3 and isinstance(name, str):
            name = _str_to_bytes(name)

        try:
            ret = _lib.caca_set_frame_name(self, name)
        except ctypes.ArgumentError:
            raise CanvasError("Specified name is invalid")
        else:
            err = ctypes.c_int.in_dll(_lib, "errno")
            if err.value == errno.ENOMEM:
                raise CanvasError("Not enough memory to allocate new frame")
            else:
                return ret

    def create_frame(self, idx):
        """ Add a frame to a canvas.

            idx -- the index where to insert the new frame
        """
        _lib.caca_create_frame.argtypes = [_Canvas, ctypes.c_int]
        _lib.caca_create_frame.restype  = ctypes.c_int

        try:
            ret = _lib.caca_create_frame(self, idx)
        except ctypes.ArgumentError:
            raise CanvasError("specified index is invalid")
        else:
            err = ctypes.c_int.in_dll(_lib, "errno")
            if err.value == errno.ENOMEM:
                raise CanvasError("Not enough memory to allocate new frame")
            else:
                return ret

    def free_frame(self, idx):
        """ Remove a frame from a canvas.

            idx -- the index of the frame to delete
        """
        _lib.caca_free_frame.argtypes = [_Canvas, ctypes.c_int]
        _lib.caca_free_frame.restype  = ctypes.c_int

        try:
            ret = _lib.caca_free_frame(self, idx)
        except ctypes.ArgumentError:
            raise CanvasError("specified index is invalid")
        else:
            err = ctypes.c_int.in_dll(_lib, "errno")
            if err.value == errno.EINVAL:
                raise CanvasError("Requested frame is out of range, or attempt"
                                  " to delete the last frame of the canvas")
            else:
                return ret

    def import_from_memory(self, data, fmt):
        """ Import a memory buffer into a canvas.

            data -- a memory area containing the data to be loaded into
                    the canvas
            fmt  -- a string describing the input format

            Valid values for format are:
              - "": attempt to autodetect the file format.
              - caca: import native libcaca files.
              - text: import ASCII text files.
              - ansi: import ANSI files.
              - utf8: import UTF-8 files with ANSI colour codes.
        """

        _lib.caca_import_canvas_from_memory.argtypes = [
                Canvas, ctypes.c_char_p,
                ctypes.c_size_t, ctypes.c_char_p
            ]
        _lib.caca_import_canvas_from_memory.restype  = ctypes.c_int

        if _PYTHON3 and isinstance(data, str):
            data = _str_to_bytes(data)
        if _PYTHON3 and isinstance(fmt, str):
            fmt = _str_to_bytes(fmt)

        length = ctypes.c_size_t(len(data))

        try:
            ret = _lib.caca_import_canvas_from_memory(self, data, length, fmt)
        except ctypes.ArgumentError:
            raise CanvasError("Given data are invalid")
        else:
            err = ctypes.c_int.in_dll(_lib, "errno")
            if ret == -1:
                if err.value == errno.ENOMEM:
                    raise CanvasError("Not enough memory to allocate canvas")
                elif err.value == errno.EINVAL:
                    raise CanvasError("Invalid format requested")
            else:
                return ret

    def import_from_file(self, filename, fmt):
        """ Import a file into a canvas.

            filename -- the name of the file to load
            fmt      -- a string describing the input format

            Valid values for format are:
              - "": attempt to autodetect the file format.
              - caca: import native libcaca files.
              - text: import ASCII text files.
              - ansi: import ANSI files.
              - utf8: import UTF-8 files with ANSI colour codes.
        """
        _lib.caca_import_canvas_from_file.argtypes = [
            _Canvas, ctypes.c_char_p, ctypes.c_char_p
        ]
        _lib.caca_import_canvas_from_file.restype  = ctypes.c_int

        if _PYTHON3 and isinstance(filename, str):
            filename = _str_to_bytes(filename)
        if _PYTHON3 and isinstance(fmt, str):
            fmt = _str_to_bytes(fmt)

        try:
            ret = _lib.caca_import_canvas_from_file(self, filename, fmt)
        except ctypes.ArgumentError:
            raise CanvasError("Specified filename is invalid")
        else:
            err = ctypes.c_int.in_dll(_lib, "errno")
            if ret == -1:
                if err.value == errno.ENOSYS:
                    raise CanvasError("File access is not implemented on this"
                                      " system")
                elif err.value == errno.ENOMEM:
                    raise CanvasError("Not enough memory to allocate canvas")
                elif err.value == errno.EINVAL:
                    raise CanvasError("Invalid format requested")
            else:
                return ret

    def import_area_from_memory(self, x, y, data, fmt):
        """ Import a memory buffer into a canvas area.

            x    -- the leftmost coordinate of the area to import to
            y    -- the topmost coordinate of the area to import to
            data -- a memory area containing the data to be loaded into
                    the canvas
            fmt  -- a string describing the input format

            Valid values for format are:
              - "": attempt to autodetect the file format.
              - caca: import native libcaca files.
              - text: import ASCII text files.
              - ansi: import ANSI files.
              - utf8: import UTF-8 files with ANSI colour codes.
        """
        length = ctypes.c_size_t(len(data))

        _lib.caca_import_area_from_memory.argtypes = [
                _Canvas, ctypes.c_int, ctypes.c_int,
                ctypes.c_char_p, ctypes.c_size_t, ctypes.c_char_p
            ]
        _lib.caca_import_area_from_memory.restype  = ctypes.c_int

        if _PYTHON3 and isinstance(data, str):
            data = _str_to_bytes(data)
        if _PYTHON3 and isinstance(fmt, str):
            fmt = _str_to_bytes(fmt)

        try:
            ret = _lib.caca_import_area_from_memory(self, x, y,
                                                    data, length, fmt)
        except ctypes.ArgumentError:
            raise CanvasError("Specified coordinate X or Y is invalid")
        else:
            if ret == -1:
                err = ctypes.c_int.in_dll(_lib, "errno")
                if err.value == errno.EINVAL:
                    raise CanvasError("Unsupported format requested or"
                                      " invalid coordinates")
                elif err.value == errno.ENOMEM:
                    raise CanvasError("Not enough memory to allocate canvas")
            else:
                return ret

    def import_area_from_file(self, x, y, filename, fmt):
        """ Import a file into a canvas area.

            x        -- the leftmost coordinate of the area to import to
            y        -- the topmost coordinate of the area to import to
            filename -- the name of the file to be load
            fmt      -- a string describing the input format

            Valid values for format are:
              - "": attempt to autodetect the file format.
              - caca: import native libcaca files.
              - text: import ASCII text files.
              - ansi: import ANSI files.
              - utf8: import UTF-8 files with ANSI colour codes.
        """
        _lib.caca_import_area_from_file.argtypes = [
                _Canvas, ctypes.c_int, ctypes.c_int,
                ctypes.c_char_p, ctypes.c_char_p
            ]
        _lib.caca_import_area_from_file.restype  = ctypes.c_int

        if _PYTHON3 and isinstance(filename, str):
            filename = _str_to_bytes(filename)
        if _PYTHON3 and isinstance(fmt, str):
            fmt = _str_to_bytes(fmt)

        try:
            ret = _lib.caca_import_area_from_file(self, x, y, filename, fmt)
        except ctypes.ArgumentError:
            raise CanvasError("Specified coordinate X or Y is invalid")
        else:
            if ret == -1:
                err = ctypes.c_int.in_dll(_lib, "errno")
                if err.value == errno.ENOSYS:
                    raise CanvasError("File access is not implemented on this"
                                      " system")
                elif err.value == errno.ENOMEM:
                    raise CanvasError("Not enough memory to allocate canvas")
                elif err.value == errno.EINVAL:
                    raise CanvasError("Unsupported format requested or"
                                      " invalid coordinates")
            else:
                return ret

    def export_to_memory(self, fmt):
        """ Export a canvas into a foreign format.

            fmt -- a string describing the output format

            Valid values for format are:
              - caca: export native libcaca files.
              - ansi: export ANSI art (CP437 charset with ANSI colour codes).
              - html: export an HTML page with CSS information.
              - html3: export an HTML table that should be compatible with
                       most navigators, including textmode ones.
              - irc: export UTF-8 text with mIRC colour codes.
              - ps: export a PostScript document.
              - svg: export an SVG vector image.
              - tga: export a TGA image.
        """
        p_size_t = ctypes.POINTER(ctypes.c_size_t)
        _lib.caca_export_canvas_to_memory.argtypes = [
                _Canvas, ctypes.c_char_p, p_size_t
            ]
        _lib.caca_export_canvas_to_memory.restype  = ctypes.POINTER(
                                                        ctypes.c_char_p)

        p = ctypes.c_size_t()

        if _PYTHON3 and isinstance(fmt, str):
            fmt = _str_to_bytes(fmt)

        try:
            ret = _lib.caca_export_canvas_to_memory(self, fmt, p)
        except ctypes.ArgumentError:
            raise CanvasError("Invalid format requested")
        else:
            if not ret:
                err = ctypes.c_int.in_dll(_lib, "errno")
                if err.value == errno.EINVAL:
                    raise CanvasError("Invalid format requested")
                elif err.value == errno.ENOMEM:
                    raise CanvasError("Not enough memory to allocate output"
                                      " buffer")
            else:
                if _PYTHON3:
                    return _bytes_to_str(ctypes.string_at(ret, p.value))
                else:
                    return ctypes.string_at(ret, p.value)

    def export_area_to_memory(self, x, y, width, height, fmt):
        """ Export a canvas portion into a foreign format.

            x       -- the leftmost coordinate of the area to export
            y       -- the topmost coordinate of the area to export
            width   -- the width of the area to export
            height  -- the height of the area to export
            fmt     -- a string describing the output format

            Valid values for format are:
              - caca: export native libcaca files.
              - ansi: export ANSI art (CP437 charset with ANSI colour codes).
              - html: export an HTML page with CSS information.
              - html3: export an HTML table that should be compatible with
                       most navigators, including textmode ones.
              - irc: export UTF-8 text with mIRC colour codes.
              - ps: export a PostScript document.
              - svg: export an SVG vector image.
              - tga: export a TGA image.
        """
        p_size_t = ctypes.POINTER(ctypes.c_size_t)

        _lib.caca_export_area_to_memory.argtypes = [
                _Canvas, ctypes.c_int, ctypes.c_int, ctypes.c_int,
                ctypes.c_int, ctypes.c_char_p, p_size_t
            ]
        _lib.caca_export_area_to_memory.restype  = ctypes.POINTER(ctypes.c_char_p)

        p = ctypes.c_size_t()

        if _PYTHON3 and isinstance(fmt, str):
            fmt = _str_to_bytes(fmt)

        try:
            ret = _lib.caca_export_area_to_memory(self, x, y, width, height,
                                                  fmt, p)
        except ctypes.ArgumentError:
            raise CanvasError("Requested area coordinate is invalid")
        else:
            if not ret:
                err = ctypes.c_int.in_dll(_lib, "errno")
                if err.value == errno.EINVAL:
                    raise CanvasError("Invalid format requested")
                elif err.value == errno.ENOMEM:
                    raise CanvasError("Not enough memory to allocate output"
                                      " buffer")
            else:
                if _PYTHON3:
                    return _bytes_to_str(ctypes.string_at(ret, p.value))
                else:
                    return ctypes.string_at(ret, p.value)

    def set_figfont(self, filename):
        """ Load a figfont and attach it to a canvas.

            filename    -- the figfont file to load.
        """
        _lib.caca_canvas_set_figfont.argtypes = [_Canvas, ctypes.c_char_p]
        _lib.caca_canvas_set_figfont.restype  = ctypes.c_int

        if _PYTHON3 and isinstance(filename, str):
            filename = _str_to_bytes(filename)

        return _lib.caca_canvas_set_figfont(self, filename)

    def put_figchar(self, ch):
        """ Paste a character using the current figfont.

            ch  -- the character to paste
        """
        _lib.caca_put_figchar.argtypes = [_Canvas, ctypes.c_uint32]
        _lib.caca_put_figchar.restype  = ctypes.c_int

        if _PYTHON3 and isinstance(ch, str):
            ch = _str_to_bytes(ch)

        try:
            ch = ord(ch)
        except TypeError:
            ch = utf8_to_utf32(ch)

        return _lib.caca_put_figchar(self, ch)

    def flush_figlet(self):
        """ Flush the figlet context
        """
        _lib.caca_flush_figlet.argtypes = [_Canvas]
        _lib.caca_flush_figlet.restype  = ctypes.c_int

        return _lib.caca_flush_figlet(self)

    def render(self, font, buf, width, height, pitch):
        """ Render the canvas onto an image buffer.

            font    -- a Font() object
            buf     -- the image buffer
            width   -- the width (in pixels) of the image
            heigth  -- the height (in pixels) of the image
            pitch   -- the pitch (in bytes) of the image
        """
        _lib.caca_render_canvas.argtypes = [
            _Canvas, _Font, ctypes.c_char_p,
            ctypes.c_int, ctypes.c_int, ctypes.c_int
        ]
        _lib.caca_render_canvas.restype  = ctypes.c_int

        return _lib.caca_render_canvas(self, font, buf, width, height, pitch)

class NullCanvas(_Canvas):
    """ Represent a NULL canvas_t, eg to use as canvas mask for blit operations.
    """
    def __str__(self):
        return "<NullCanvas>"

class CanvasError(Exception):
    pass




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# driver        libcaca test drivers program
# Copyright (c) 2010 Alex Foulon <alxf@lavabit.com>
#
# This file is a Python port of "examples/driver.c"
# which is:
# Copyright (c) 2006-2010 Sam Hocevar <sam@hocevar.net>
#               All Rights Reserverd
#
# This library is free software. It comes without any warranty, to
# the extent permitted by applicable law. You can redistribute it
# and/or modify it under the terms of the Do What the Fuck You Want
# to Public License, Version 2, as published by Sam Hocevar. See
# http://www.wtfpl.net/ for more details.
#

import sys

import caca
from caca.canvas import Canvas, CanvasError
from caca.display import Display, DisplayError, Event

def main():
    """ Main function. """

    lst = caca.get_display_driver_list()
    cur = 0

    try:
        cv = Canvas(0, 0)
        dp = Display(cv)
    except (CanvasError, DisplayError) as err:
        sys.stderr.write("%s\n" % err)
        sys.exit(127)

    cv.set_color_ansi(caca.COLOR_WHITE, caca.COLOR_BLACK)

    while True:
        cv.put_str(1, 0, "Available drivers:")
        cur_driver = dp.get_driver()
        n = 0
        for driver, desc in lst:
            if driver == cur_driver:
                cv.put_str(2, n + 2, "%s %s (%s)" % ('*', driver, desc))
            else:
                cv.put_str(2, n + 2, "%s %s (%s)" % (' ', driver, desc))
            n += 1

        cv.put_str(2, n + 3, "Switching driver in 5 seconds")
        dp.refresh()

        if dp.get_event(caca.EVENT_KEY_PRESS, Event(), 5000000):
            break

        cur += 1
        if cur < len(lst) and lst[cur][0] == "raw":
            cur += 1
        if cur >= len(lst):
            cur = 0

        dp.set_driver(lst[cur][0])

if __name__ == "__main__":
    main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# img2txt       image to text converter
# Copyright (c) 2010 Alex Foulon <alxf@lavabit.com>
#
# This file is a Python port of "src/img2txt.c"
# which is:
# Copyright (c) 2006 Sam Hocevar <sam@hocevar.net>
#               2007 Jean-Yves Lamoureux <jylam@lnxscene.org>
#               All Rights Reserverd
#
# This library is free software. It comes without any warranty, to
# the extent permitted by applicable law. You can redistribute it
# and/or modify it under the terms of the Do What the Fuck You Want
# to Public License, Version 2, as published by Sam Hocevar. See
# http://www.wtfpl.net/ for more details.
#

import getopt
import os
import sys

import caca
from caca.canvas import Canvas, CanvasError
from caca.dither import Dither, DitherError

RMASK = 0x00ff0000
GMASK = 0x0000ff00
BMASK = 0x000000ff
AMASK = 0xff000000
BPP = 32
DEPTH = 4

HELP_MSG = """\
Usage: %s [OPTIONS]... <IMAGE>
Convert IMAGE to any text based available format.
Example: %s -w 80 -f ansi ./caca.png

Options:
  -h, --help                      This help
  -v, --version                   Version of the program
  -W, --width=WIDTH               Width of resulting image
  -H, --height=HEIGHT             Height of resulting image
  -x, --font-width=WIDTH          Width of output font
  -y, --font-height=HEIGHT        Height of output font
  -b, --brightness=BRIGHTNESS     Brightness of resulting image
  -c, --contrast=CONTRAST         Contrast of resulting image
  -g, --gamma=GAMMA               Gamma of resulting image
  -d, --dither=DITHER             Dithering algorithm to use
  -f, --format=FORMAT             Format of the resulting image
  -C, --charset=CHARSET           Charset of the resulting image

DITHER list:
  - none: no dithering
  - ordered2: 2x2 ordered dithering
  - ordered4: 4x4 ordered dithering
  - ordered8: 8x8 orederd dithering
  - random: random dithering
  - fstein: Floyd-Steinberg dithering

FORMAT list:
  - caca: native libcaca format
  - ansi: ANSI
  - utf8: UTF-8 with ANSI escape codes
  - utf8cr: UTF-8 with ANSI escape codes and MS-DOS \\r
  - html: HTML
  - html3: backwards-compatible HTML
  - bbfr: BBCode (French)
  - irc: IRC with mIRC colours
  - ps: PostScript document
  - svg: SVG vector image
  - tga: TGA image
  - troff: troff source

CHARSET list:
  - ascii: use only ascii character
  - shades: use unicode character
  - blocks: use unicode quarter-cell combinations
""" % (os.path.basename(sys.argv[0]), os.path.basename(sys.argv[0]))

VERSION_MSG="""\
img2txt Copyright 2006-2007 Sam Hocevar and Jean-Yves Lamoureux
        Copyright 2010      Alex Foulon
Internet: <sam@hocevar.net> <jylam@lnxscene.org>
          <alxf@lavabit.com> version: %s

img2txt, along with its documentation, may be freely copied and distributed.

The latest version of img2txt is available from the web site,
        http://caca.zoy.org/wiki/libcaca in the libcaca package.
""" % caca.get_version()

def main():
    """ Main script function.
    """

    #init options vars
    width = None
    height = None
    font_width = 6
    font_height = 10
    brightness = None
    contrast = None
    gamma = None
    ditalgo = None
    exformat = "ansi"
    charset = None

    #define long and short options
    shortopts = "hvW:H:x:y:b:c:g:d:f:C:"
    longopts = [
        "help",
        "version",
        "width=",
        "height=",
        "font-width=",
        "font-height=",
        "brightness=",
        "contrast=",
        "gamma=",
        "dither=",
        "format=",
        "charset=",
    ]

    #parse command line
    try:
        opts, args = getopt.getopt(sys.argv[1:], shortopts, longopts)
    except getopt.GetoptError, err:
        sys.stderr.write("%s\n" % str(err))
        sys.exit(127)

    for o, a in opts:
        if o in ('-h', '--help'):
            sys.stdout.write(HELP_MSG)
            sys.exit(0)
        elif o in ('-v', '--version'):
            sys.stdout.write("%s\n" % VERSION_MSG)
            sys.exit(0)
        elif o in ('-W', '--width'):
            width = int(a)
        elif o in ('-H', '--height'):
            height = int(a)
        elif o in ('-x', '--font-width'):
            font_width = int(a)
        elif o in ('-y', '--font-height'):
            font_height = int(a)
        elif o in ('-b', '--brightness'):
            brightness = float(a)
        elif o in ('-c', '--contrast'):
            contrast = float(a)
        elif o in ('-g', '--gamma'):
            gamma = float(a)
        elif o in ('-d', '--dither'):
            ditalgo = a
        elif o in ('-f', '--format'):
            exformat = a
        elif o in ('-C', '--charset'):
            charset = a

    if not args:
        sys.stderr.write("%s: Missing argument...\n" % os.path.basename(sys.argv[0]))
        sys.exit(127)
    else:
        try:
            img = Image.open(args[0])
            img.load()
        except IOError, err:
            sys.stderr.write("%s\n" % err)
            sys.exit(127)

    if not width and not height:
        width = 60
        height = width * img.size[1] * font_width / img.size[0] / font_height
    elif width and not height:
        height = width * img.size[1] * font_width / img.size[0] / font_height
    elif not width and height:
        width = height * img.size[0] * font_height / img.size[1] / font_width

    #init canvas
    try:
        cv = Canvas(width, height)
    except CanvasError, err:
        sys.stderr.write("%s\n" % err)
        sys.exit(127)

    cv.set_color_ansi(caca.COLOR_DEFAULT, caca.COLOR_TRANSPARENT)

    #init dither
    try:
        #convert rgb to rgba
        if img.mode == 'RGB':
            img = img.convert('RGBA')
        #reorder rgba
        if img.mode == 'RGBA':
            r, g, b, a = img.split()
            img = Image.merge("RGBA", (b, g, r, a))

        dit = Dither(BPP, img.size[0], img.size[1], DEPTH * img.size[0],
                        RMASK, GMASK, BMASK, AMASK)
    except DitherError, err:
        sys.stderr.write("%s\n" % err)
        sys.exit(127)

    #set dither algorithm
    if ditalgo:
        dit.set_algorithm(ditalgo)

    #set brightness
    if brightness:
        dit.set_brightness(brightness)

    #set gamma
    if gamma:
        dit.set_gamma(gamma)

    #set contrast
    if contrast:
        dit.set_contrast(contrast)

    #set charset
    if charset:
        dit.set_charset(charset)

    #create dither
    dit.bitmap(cv, 0, 0, width, height, img.tostring())

    #print export to screen
    sys.stdout.write("%s" % cv.export_to_memory(exformat))

if __name__ == "__main__":
    #Import PIL package
    try:
        from PIL import Image
    except ImportError, err:
        sys.stderr.write("You need to install PIL module !\n")
        sys.exit(2)

    main()




# -*- coding: utf-8 -*-
#
# libcaca       Colour ASCII-Art library
#               Python language bindings
# Copyright (c) 2010 Alex Foulon <alxf@lavabit.com>
#               All Rights Reserved
#
# This library is free software. It comes without any warranty, to
# the extent permitted by applicable law. You can redistribute it
# and/or modify it under the terms of the Do What the Fuck You Want
# to Public License, Version 2, as published by Sam Hocevar. See
# http://www.wtfpl.net/ for more details.
#

""" Libcaca Python bindings """

import ctypes
import errno

from caca import _lib, _PYTHON3, _str_to_bytes

class _Font(object):
    """ Model for Font object.
    """
    def __init__(self):
        self._font = 0

    def from_param(self):
        """ Required by ctypes module to call object as parameter of
            a C function.
        """
        return self._font

    def __del__(self):
        if hasattr(self, "_font"):
            if self._font > 0:
                self._free()

    def __str__(self):
        return "<CacaFont>"

    def _free(self):
        """ Free a libcaca font.
        """
        _lib.caca_free_font.argtypes = [_Font]
        _lib.caca_free_font.restype = ctypes.c_int

        return _lib.caca_free_font(self)

class Font(_Font):
    """ Font object, methods are libcaca functions with caca_font_t as first
        argument.
    """
    def __init__(self, font, size=0):
        """ Font constructor

            font    -- the memory area containing the font or its name
            size    -- the size of the memory area, or 0 if the font name is given
        """
        if size == 0:
            _lib.caca_load_font.argtypes = [ctypes.c_char_p, ctypes.c_int]
        else:
            raise FontError("Unsupported method")

        _lib.caca_load_font.restype = ctypes.c_int

        if _PYTHON3:
            font = _str_to_bytes(font)

        self._font = _lib.caca_load_font(font, size)
        if self._font == 0:
            err = ctypes.c_int.in_dll(_lib, "errno")
            if err.value == errno.ENOENT:
                raise FontError("Requested built-in font does not exist")
            elif err.value == errno.EINVAL:
                raise FontError("Invalid font data in memory area")
            elif err.value == errno.ENOMEM:
                raise FontError("Not enough memory to allocate font structure")

    def get_width(self):
        """ Get a font's standard glyph width.
        """
        _lib.caca_get_font_width.argtypes = [_Font]
        _lib.caca_get_font_width.restype  = ctypes.c_int

        return _lib.caca_get_font_width(self)

    def get_height(self):
        """ Get a font's standard glyph height.
        """
        _lib.caca_get_font_height.argtypes = [_Font]
        _lib.caca_get_font_height.restype  = ctypes.c_int

        return _lib.caca_get_font_height(self)

    def get_blocks(self):
        """ Get a font's list of supported glyphs.
        """
        raise FontError("Not Implemented")

class FontError(Exception):
    pass

